@@ -17,9 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 #include <config.h>
-#include "gdk-pixdata.h"
 
 #include "gdk-pixbuf-private.h"
+#include "gdk-pixdata.h"
+#include "gdk-pixbuf-alias.h"
 #include <string.h>
 
 #define APPEND g_string_append_printf
@@ -289,6 +290,13 @@ rl_encode_rgbx (guint8 *bp,	/* dest buffer */
   return bp;
 }
 
+/* Used as the destroy notification function for gdk_pixbuf_new() */
+static void
+free_buffer (guchar *pixels, gpointer data)
+{
+	g_free (pixels);
+}
+
 /**
  * gdk_pixdata_from_pixbuf:
  * @pixdata: a #GdkPixdata to fill.
@@ -327,16 +335,36 @@ gdk_pixdata_from_pixbuf (GdkPixdata      *pixdata,
     {
       guint pad, n_bytes = rowstride * height;
       guint8 *img_buffer_end, *data;
+      GdkPixbuf *buf = NULL;
 
+      if (n_bytes % bpp != 0) 
+	{
+	  rowstride = pixbuf->width * bpp;
+	  n_bytes = rowstride * height;
+	  data = g_malloc (n_bytes);
+	  buf = gdk_pixbuf_new_from_data (data,
+					  GDK_COLORSPACE_RGB,
+					  pixbuf->has_alpha, 8,
+					  pixbuf->width,
+					  pixbuf->height,
+					  rowstride,
+					  free_buffer, NULL);
+	  gdk_pixbuf_copy_area (pixbuf, 0, 0, pixbuf->width, pixbuf->height,
+				buf, 0, 0);
+	}
+      else
+	buf = (GdkPixbuf *)pixbuf;
       pad = rowstride;
       pad = MAX (pad, 130 + n_bytes / 127);
       data = g_new (guint8, pad + n_bytes);
       free_me = data;
       img_buffer = data;
       img_buffer_end = rl_encode_rgbx (img_buffer,
-				       pixbuf->pixels, pixbuf->pixels + n_bytes,
+				       buf->pixels, buf->pixels + n_bytes,
 				       bpp);
       length = img_buffer_end - img_buffer;
+      if (buf != pixbuf)
+	g_object_unref (buf);
     }
   else
     {
@@ -893,3 +921,6 @@ gdk_pixbuf_new_from_inline (gint          data_length,
 
   return gdk_pixbuf_from_pixdata (&pixdata, copy_pixels, error);
 }
+
+#define __GDK_PIXDATA_C__
+#include "gdk-pixbuf-aliasdef.c"
