@@ -344,8 +344,8 @@ gdk_pixbuf__png_image_load (FILE *f, GError **error)
 static void png_error_callback  (png_structp png_read_ptr,
                                  png_const_charp error_msg);
 
-static void png_warning_callback(png_structp png_read_ptr,
-                                 png_const_charp warning_msg);
+static void png_warning_callback (png_structp png_read_ptr,
+                                  png_const_charp warning_msg);
 
 /* Called at the start of the progressive load */
 static void png_info_callback   (png_structp png_read_ptr,
@@ -739,8 +739,8 @@ png_error_callback(png_structp png_read_ptr,
 }
 
 static void
-png_warning_callback(png_structp png_read_ptr,
-                     png_const_charp warning_msg)
+png_warning_callback (png_structp png_read_ptr,
+                      png_const_charp warning_msg)
 {
         LoadContext* lc;
         
@@ -861,9 +861,7 @@ static gboolean real_save_png (GdkPixbuf        *pixbuf,
                                        return FALSE;
                                }
                        } else {
-                               g_warning ("Bad option name '%s' passed to PNG saver",
-                                          *kiter);
-                               return FALSE;
+                               g_warning ("Unrecognized parameter (%s) passed to PNG saver.", *kiter);
                        }
 
                        ++kiter;
@@ -917,8 +915,10 @@ static gboolean real_save_png (GdkPixbuf        *pixbuf,
                                           error,
                                           png_simple_error_callback,
                                           png_simple_warning_callback);
-
-       g_return_val_if_fail (png_ptr != NULL, FALSE);
+       if (png_ptr == NULL) {
+	       success = FALSE;
+	       goto cleanup;
+       }
 
        info_ptr = png_create_info_struct (png_ptr);
        if (info_ptr == NULL) {
