@@ -23,11 +23,6 @@
 /*
  * Some NOTES about the TGA loader (2001/06/07, nikke@swlibero.org)
  *
- * - The module doesn't currently provide support for TGA images where the
- *   order of the pixels isn't left-to-right and top-to-bottom.  I plan to
- *   add support for those files as soon as I get one of them.  I haven't
- *   run into one yet.  (And I don't seem to be able to create it with GIMP.)
- *
  * - The TGAFooter isn't present in all TGA files.  In fact, there's an older
  *   format specification, still in use, which doesn't cover the TGAFooter.
  *   Actually, most TGA files I have are of the older type.  Anyway I put the 
@@ -509,8 +504,9 @@ static guint parse_rle_data_pseudocolor(TGAContext *ctx)
 		}
 	}
 
-	if (ctx->pbuf_bytes_done == ctx->pbuf_bytes)
+	if (ctx->pbuf_bytes_done == ctx->pbuf_bytes) 
 		ctx->done = TRUE;
+	
 	return n;
 }
 
@@ -635,19 +631,16 @@ static guint parse_rle_data_grayscale(TGAContext *ctx)
 
 static gboolean parse_rle_data(TGAContext *ctx, GError **err)
 {
+	guint rows = 0;
 	guint count = 0;
-	guint pbuf_count = 0;
 	guint bytes_done_before = ctx->pbuf_bytes_done;
-	if (ctx->hdr->type == TGA_TYPE_RLE_PSEUDOCOLOR) {
+
+	if (ctx->hdr->type == TGA_TYPE_RLE_PSEUDOCOLOR)
 		count = parse_rle_data_pseudocolor(ctx);
-		pbuf_count = count * ctx->pbuf->n_channels;
-	} else if (ctx->hdr->type == TGA_TYPE_RLE_TRUECOLOR) {
+	else if (ctx->hdr->type == TGA_TYPE_RLE_TRUECOLOR)
 		count = parse_rle_data_truecolor(ctx);
-		pbuf_count = count;
-	} else if (ctx->hdr->type == TGA_TYPE_RLE_GRAYSCALE) {
+	else if (ctx->hdr->type == TGA_TYPE_RLE_GRAYSCALE)
 		count = parse_rle_data_grayscale(ctx);
-		pbuf_count = count * (ctx->pbuf->n_channels == 4 ? 2 : 3);
-	}
 
 	if (ctx->hdr->flags & TGA_ORIGIN_RIGHT) {
 		guchar *row = ctx->pbuf->pixels + (bytes_done_before / ctx->pbuf->rowstride) * ctx->pbuf->rowstride;
@@ -665,16 +658,19 @@ static gboolean parse_rle_data(TGAContext *ctx, GError **err)
 		 * perfect, but doing it during the rle decoding in place
 		 * is considerably more work. 
 		 */
-		if (!(ctx->hdr->flags & TGA_ORIGIN_UPPER))
+		if (!(ctx->hdr->flags & TGA_ORIGIN_UPPER)) {
 			pixbuf_flip_vertically (ctx->pbuf);
+			ctx->hdr->flags |= TGA_ORIGIN_UPPER;
+		}
+
 	}
-
+		
+	rows = ctx->pbuf_bytes_done / ctx->pbuf->rowstride - bytes_done_before / ctx->pbuf->rowstride;
 	if (ctx->ufunc)
-		(*ctx->ufunc) (ctx->pbuf, 0, ctx->pbuf_bytes_done / ctx->pbuf->rowstride,
-			       ctx->pbuf->width, pbuf_count / ctx->pbuf->rowstride,
+		(*ctx->ufunc) (ctx->pbuf, 0, bytes_done_before / ctx->pbuf->rowstride,
+			       ctx->pbuf->width, rows,
 			       ctx->udata);
 
-
 	return TRUE;
 }
 
@@ -940,6 +936,13 @@ static gboolean gdk_pixbuf__tga_stop_load(gpointer data, GError **err)
 	TGAContext *ctx = (TGAContext *) data;
 	g_return_val_if_fail(ctx != NULL, FALSE);
 
+	if (!(ctx->hdr->flags & TGA_ORIGIN_UPPER) && ctx->run_length_encoded) {
+		pixbuf_flip_vertically (ctx->pbuf);
+		if (ctx->ufunc)
+			(*ctx->ufunc) (ctx->pbuf, 0, 0,
+				       ctx->pbuf->width, ctx->pbuf->height,
+			       	       ctx->udata);
+	}
 	if (ctx->hdr)
 	  g_free (ctx->hdr);
 	if (ctx->cmap) {
