@@ -1670,6 +1670,8 @@ _gdk_directfb_keyboard_init (void)
 
 	if (!keyboard)
 		return;
+    if( directfb_keymap )    
+            return;
 
 	keyboard->GetDescription (keyboard, &desc);
 	_gdk_display->keymap=g_object_new (gdk_keymap_get_type (), NULL);
@@ -1682,7 +1684,6 @@ _gdk_directfb_keyboard_init (void)
 
 	length = directfb_max_keycode - desc.min_keycode + 1;
 
-	g_assert (directfb_keymap == NULL);
 
 	directfb_keymap = g_new0 (guint, 4 * length);
 
@@ -1940,31 +1941,40 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
 
 		if (directfb_keymap[index + i + 2 * group] != GDK_VoidSymbol)
 		{
+			if (keyval)
 			*keyval = directfb_keymap[index + i + 2 * group];
 
 			if (group && directfb_keymap[index + i] == *keyval)
 			{
+				if (effective_group)
 				*effective_group = 0;
+                if(consumed_modifiers)
 				*consumed_modifiers = 0;
 			}
 			else
 			{
+				if (effective_group)
 				*effective_group = group;
+                if(consumed_modifiers)
 				*consumed_modifiers = GDK_MOD2_MASK;
 			}
-
+			if (level)
 			*level = i;
 
 			if (i && directfb_keymap[index + 2 * *effective_group] != *keyval)
+                if(consumed_modifiers)
 				*consumed_modifiers |= GDK_SHIFT_MASK;
 
 			return TRUE;
 		}
 	}
-
+	if (keyval)
 	*keyval             = 0;
+	if (effective_group)
 	*effective_group    = 0;
+	if (level)
 	*level              = 0;
+    if(consumed_modifiers)
 	*consumed_modifiers = 0;
 
 	return FALSE;
