@@ -62,36 +62,8 @@ static void         gdk_drawable_real_draw_pixbuf            (GdkDrawable  *draw
 							      gint          x_dither,
 							      gint          y_dither);
      
-static void gdk_drawable_class_init (GdkDrawableClass *klass);
 
-GType
-gdk_drawable_get_type (void)
-{
-  static GType object_type = 0;
-
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (GdkDrawableClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) gdk_drawable_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (GdkDrawable),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) NULL,
-      };
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "GdkDrawable",
-                                            &object_info, 
-					    G_TYPE_FLAG_ABSTRACT);
-    }  
-
-  return object_type;
-}
+G_DEFINE_ABSTRACT_TYPE (GdkDrawable, gdk_drawable, G_TYPE_OBJECT)
 
 static void
 gdk_drawable_class_init (GdkDrawableClass *klass)
@@ -104,6 +76,11 @@ gdk_drawable_class_init (GdkDrawableClass *klass)
   klass->draw_pixbuf = gdk_drawable_real_draw_pixbuf;
 }
 
+static void
+gdk_drawable_init (GdkDrawable *drawable)
+{
+}
+
 /* Manipulation of drawables
  */
 
@@ -774,13 +751,17 @@ gdk_draw_pixbuf (GdkDrawable     *drawable,
   g_return_if_fail (gc == NULL || GDK_IS_GC (gc));
   g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
 
+  if (width == 0 || height == 0)
+    return;
+
   if (width == -1)
     width = gdk_pixbuf_get_width (pixbuf);
   if (height == -1)
     height = gdk_pixbuf_get_height (pixbuf);
 
   GDK_DRAWABLE_GET_CLASS (drawable)->draw_pixbuf (drawable, gc, pixbuf,
-						  src_x, src_y, dest_x, dest_y, width, height,
+						  src_x, src_y, dest_x, dest_y,
+                                                  width, height,
 						  dither, x_dither, y_dither);
 }
 
@@ -1012,7 +993,7 @@ gdk_draw_trapezoids (GdkDrawable    *drawable,
       cairo_move_to (cr, trapezoids[i].x11, trapezoids[i].y1);
       cairo_line_to (cr, trapezoids[i].x21, trapezoids[i].y1);
       cairo_line_to (cr, trapezoids[i].x22, trapezoids[i].y2);
-      cairo_line_to (cr, trapezoids[i].x21, trapezoids[i].y2);
+      cairo_line_to (cr, trapezoids[i].x12, trapezoids[i].y2);
       cairo_close_path (cr);
     }
 
