@@ -2,7 +2,7 @@
  *
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
  * Copyright (C) 1998-2002 Tor Lillqvist
- * Copyright (C) 2005 Imendio AB
+ * Copyright (C) 2005-2006 Imendio AB
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -250,7 +250,6 @@ poll_func (GPollFD *ufds, guint nfds, gint timeout_)
     {
       if ([event type] == NSApplicationDefined)
 	{
-
 	  pthread_mutex_lock (&pollfd_mutex);
 
 	  for (i = 0; i < n_pollfds; i++)
@@ -282,6 +281,7 @@ poll_func (GPollFD *ufds, guint nfds, gint timeout_)
   if (n_active == 0 && wakeup_pipe[1])
     {
       char c = 'A';
+
       write (wakeup_pipe[1], &c, 1);
     }
 
@@ -289,8 +289,8 @@ poll_func (GPollFD *ufds, guint nfds, gint timeout_)
     {
       ufds[0].revents = G_IO_IN;
 
-      /* FIXME: We can't assert here, what we need to to is to have a queue
-       * for events instead.
+      /* FIXME: We can't assert here, but we might need to have a
+       * queue for events instead.
        */
       /*g_assert (current_event == NULL);*/
 
@@ -369,8 +369,10 @@ gdk_keyboard_grab (GdkWindow  *window,
 
   if (_gdk_quartz_keyboard_grab_window)
     {
-      generate_grab_broken_event (_gdk_quartz_keyboard_grab_window,
-				  TRUE, FALSE, window);
+      if (_gdk_quartz_keyboard_grab_window != window)
+	generate_grab_broken_event (_gdk_quartz_keyboard_grab_window,
+				    TRUE, FALSE, window);
+      
       g_object_unref (_gdk_quartz_keyboard_grab_window);
     }
 
@@ -408,17 +410,17 @@ gdk_keyboard_grab_info_libgtk_only (GdkDisplay *display,
 }
 
 static void
-pointer_ungrab_internal (gboolean implicit)
+pointer_ungrab_internal (gboolean only_if_implicit)
 {
   if (!_gdk_quartz_pointer_grab_window)
     return;
 
-  if (pointer_grab_implicit && !implicit)
+  if (only_if_implicit && !pointer_grab_implicit)
     return;
 
   g_object_unref (_gdk_quartz_pointer_grab_window);
-
   _gdk_quartz_pointer_grab_window = NULL;
+
   /* FIXME: Send crossing events */
 }
 
@@ -563,16 +565,6 @@ apply_filters (GdkWindow  *window,
   return result;
 }
 
-/* Returns the current keyboard window */
-static GdkWindow *
-find_current_keyboard_window (void)
-{
-  if (_gdk_quartz_keyboard_grab_window && keyboard_grab_owner_events)
-    return _gdk_quartz_keyboard_grab_window;
-  
-  return current_keyboard_window;
-}
-
 /* This function checks if the passed in window is interested in the
  * event mask. If so, it's returned. If not, the event can be propagated
  * to its parent.
@@ -645,11 +637,13 @@ get_event_mask_from_ns_event (NSEvent *nsevent)
        */
       return GDK_SCROLL_MASK | GDK_BUTTON_PRESS_MASK;
     case NSLeftMouseDragged:
-      return GDK_POINTER_MOTION_MASK|GDK_POINTER_MOTION_HINT_MASK |
-	     GDK_BUTTON_MOTION_MASK|GDK_BUTTON1_MOTION_MASK;
+      return (GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |
+	      GDK_BUTTON_MOTION_MASK | GDK_BUTTON1_MOTION_MASK | 
+	      GDK_BUTTON1_MASK);
     case NSRightMouseDragged:
-      return GDK_POINTER_MOTION_MASK|GDK_POINTER_MOTION_HINT_MASK |
-	     GDK_BUTTON_MOTION_MASK|GDK_BUTTON3_MOTION_MASK;
+      return (GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |
+	      GDK_BUTTON_MOTION_MASK | GDK_BUTTON3_MOTION_MASK | 
+	      GDK_BUTTON3_MASK);
     case NSOtherMouseDragged:
       {
 	GdkEventMask mask;
@@ -659,7 +653,8 @@ get_event_mask_from_ns_event (NSEvent *nsevent)
 		GDK_BUTTON_MOTION_MASK);
 
 	if (convert_mouse_button_number ([nsevent buttonNumber]) == 2)
-	  mask |= GDK_BUTTON2_MOTION_MASK;
+	  mask |= (GDK_BUTTON2_MOTION_MASK | GDK_BUTTON2_MOTION_MASK | 
+		   GDK_BUTTON2_MASK);
 
 	return mask;
       }
@@ -709,17 +704,16 @@ _gdk_quartz_update_focus_window (GdkWindow *window,
   if (got_focus && window == current_keyboard_window)
     return;
 
-  /* FIXME: Don't do this when grabbed */
-
-  if (!got_focus)
+  /* FIXME: Don't do this when grabbed? Or make GdkQuartzWindow
+   * disallow it in the first place instead?
+   */
+  
+  if (!got_focus && window == current_keyboard_window)
     {
-      if (window == current_keyboard_window)
-	{
 	  event = create_focus_event (current_keyboard_window, FALSE);
 	  append_event (event);
 	  g_object_unref (current_keyboard_window);
 	  current_keyboard_window = NULL;
-	}
     }
 
   if (got_focus)
@@ -765,7 +759,7 @@ get_keyboard_modifiers_from_nsevent (NSEvent *nsevent)
     modifiers |= GDK_CONTROL_MASK;
   if (nsflags & NSCommandKeyMask)
     modifiers |= GDK_MOD1_MASK;
-  
+
   /* FIXME: Support GDK_BUTTON_MASK */
 
   return modifiers;
@@ -964,8 +958,15 @@ synthesize_crossing_events (GdkWindow      *window,
     }
   else
     {
-      /* Dunno where we are coming from */
-      synthesize_enter_event (window, nsevent, mode, GDK_NOTIFY_UNKNOWN);
+      /* This means we have not current_mouse_window. FIXME: Should
+       * we make sure to always set the root window instead of NULL?
+       */
+
+      /* FIXME: Figure out why this is being called with window being
+       * NULL. The check works around a crash for now.
+       */ 
+      if (window)
+	synthesize_enter_event (window, nsevent, mode, GDK_NOTIFY_UNKNOWN);
     }
   
   _gdk_quartz_update_mouse_window (window);
@@ -997,7 +998,7 @@ _gdk_quartz_send_map_events (GdkWindow *window)
 GdkWindow *
 _gdk_quartz_get_mouse_window (void)
 {
-  if (_gdk_quartz_pointer_grab_window)
+  if (_gdk_quartz_pointer_grab_window && !pointer_grab_owner_events)
     return _gdk_quartz_pointer_grab_window;
   
   return current_mouse_window;
@@ -1150,6 +1151,8 @@ find_window_for_event (NSEvent *nsevent, gint *x, gint *y)
 	
 	return real_window;
       }
+      break;
+      
     case NSMouseEntered:
       {
 	NSPoint point;
@@ -1163,9 +1166,9 @@ find_window_for_event (NSEvent *nsevent, gint *x, gint *y)
 	mouse_window = _gdk_quartz_find_child_window_by_point (toplevel, point.x, point.y, x, y);
 	
 	synthesize_crossing_events (mouse_window, GDK_CROSSING_NORMAL, nsevent, *x, *y);
-
-	break;
       }
+      break;
+
     case NSMouseExited:
       synthesize_crossing_events (_gdk_root, GDK_CROSSING_NORMAL, nsevent, *x, *y);
       break;
@@ -1174,18 +1177,13 @@ find_window_for_event (NSEvent *nsevent, gint *x, gint *y)
     case NSKeyUp:
     case NSFlagsChanged:
       {
-	GdkWindow *keyboard_window;
 	GdkEventMask event_mask;
-	GdkWindow *real_window;
 
 	if (_gdk_quartz_keyboard_grab_window && !keyboard_grab_owner_events)
 	  return _gdk_quartz_keyboard_grab_window;
 
-	keyboard_window = find_current_keyboard_window ();
 	event_mask = get_event_mask_from_ns_event (nsevent);
-	real_window = find_window_interested_in_event_mask (keyboard_window, event_mask, TRUE);
-
-	return real_window;
+	return find_window_interested_in_event_mask (current_keyboard_window, event_mask, TRUE);
       }
       break;
 
@@ -1214,24 +1212,18 @@ create_button_event (GdkWindow *window, NSEvent *nsevent,
     case NSRightMouseDown:
     case NSOtherMouseDown:
       type = GDK_BUTTON_PRESS;
-      button = convert_mouse_button_number ([nsevent buttonNumber]);
       break;
     case NSLeftMouseUp:
-      type = GDK_BUTTON_RELEASE;
-      button = 1;
-      break;
     case NSRightMouseUp:
-      type = GDK_BUTTON_RELEASE;
-      button = 3;
-      break;
     case NSOtherMouseUp:
       type = GDK_BUTTON_RELEASE;
-      button = convert_mouse_button_number ([nsevent buttonNumber]);
       break;
     default:
       g_assert_not_reached ();
     }
   
+  button = convert_mouse_button_number ([nsevent buttonNumber]);
+
   event = gdk_event_new (type);
   event->button.window = window;
   event->button.time = get_event_time (nsevent);
@@ -1308,8 +1300,6 @@ create_scroll_event (GdkWindow *window, NSEvent *nsevent, GdkScrollDirection dir
 				      &event->scroll.x_root,
 				      &event->scroll.y_root);
 
-  /* FIXME event->state; */
-  /* FIXME event->is_hint; */
   event->scroll.direction = direction;
   event->scroll.device = _gdk_display->core_pointer;
   
@@ -1388,6 +1378,12 @@ create_key_event (GdkWindow *window, NSEvent *nsevent, GdkEventType type)
   return event;
 }
 
+static GdkEventMask current_mask = 0;
+GdkEventMask _gdk_quartz_get_current_event_mask (void)
+{
+  return current_mask;
+}
+
 static gboolean
 gdk_event_translate (NSEvent *nsevent)
 {
@@ -1410,7 +1406,38 @@ gdk_event_translate (NSEvent *nsevent)
 	return TRUE;
     }
 
+  /* Catch the case where the entire app loses focus, and break any grabs. */
+  if ([nsevent type] == NSAppKitDefined)
+    {
+      if ([nsevent subtype] == NSApplicationDeactivatedEventType)
+	{
+	  if (_gdk_quartz_keyboard_grab_window)
+	    {
+	      generate_grab_broken_event (_gdk_quartz_keyboard_grab_window,
+					  TRUE, FALSE,
+					  NULL);
+	      g_object_unref (_gdk_quartz_keyboard_grab_window);
+	      _gdk_quartz_keyboard_grab_window = NULL;
+	    }
+
+	  if (_gdk_quartz_pointer_grab_window)
+	    {
+	      generate_grab_broken_event (_gdk_quartz_pointer_grab_window,
+					  FALSE, pointer_grab_implicit,
+					  NULL);
+	      g_object_unref (_gdk_quartz_pointer_grab_window);
+	      _gdk_quartz_pointer_grab_window = NULL;
+	    }
+	}
+    }
+
   window = find_window_for_event (nsevent, &x, &y);
+
+  /* FIXME: During owner_event grabs, we don't find a window when there is a
+   * click on a no-window widget, which makes popups etc still stay up. Need
+   * to figure out why that is.
+   */
+  
   if (!window)
     return FALSE;
 
@@ -1418,19 +1445,31 @@ gdk_event_translate (NSEvent *nsevent)
 
   if (result == GDK_FILTER_REMOVE)
     return TRUE;
-  
+
+  current_mask = get_event_mask_from_ns_event (nsevent);
+
   switch ([nsevent type])
     {
     case NSLeftMouseDown:
     case NSRightMouseDown:
     case NSOtherMouseDown:
-      /* Emulate implicit grab */
-      if (!_gdk_quartz_pointer_grab_window)
-	{
-	  pointer_grab_internal (window, FALSE, GDK_WINDOW_OBJECT (window)->event_mask,
-				 NULL, NULL, TRUE);
-	}
+      {
+	GdkEventMask event_mask;
 
+	/* Emulate implicit grab, when the window has both PRESS and RELEASE
+	 * in its mask, like X (and make it owner_events since that's what
+	 * implicit grabs are like).
+	 */
+	event_mask = (GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_RELEASE_MASK);
+	if (!_gdk_quartz_pointer_grab_window &&
+	    (GDK_WINDOW_OBJECT (window)->event_mask & event_mask) == event_mask)
+	  {
+	    pointer_grab_internal (window, TRUE,
+				   GDK_WINDOW_OBJECT (window)->event_mask,
+				   NULL, NULL, TRUE);
+	  }
+      }
+      
       event = create_button_event (window, nsevent, x, y);
       append_event (event);
       
@@ -1575,10 +1614,32 @@ gdk_screen_get_setting (GdkScreen   *screen,
 			const gchar *name,
 			GValue      *value)
 {
+  /* FIXME: This should be fetched from the correct preference value. See:
+     http://developer.apple.com/documentation/UserExperience/\
+     Conceptual/OSXHIGuidelines/XHIGText/chapter_13_section_2.html
+  */
   if (strcmp (name, "gtk-font-name") == 0)
     {
-      /* FIXME: This should be fetched from the correct preference value */
-      g_value_set_string (value, "Lucida Grande 13");
+      g_value_set_string (value, "Lucida Grande 12");
+      return TRUE;
+    }
+  else if (strcmp (name, "gtk-double-click-time") == 0)
+    {
+      NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+      float t;
+
+      GDK_QUARTZ_ALLOC_POOL;
+            
+      t = [defaults floatForKey:@"com.apple.mouse.doubleClickThreshold"];
+      if (t == 0.0)
+	{
+	  /* No user setting, use the default in OS X. */
+	  t = 0.5;
+	}
+
+      GDK_QUARTZ_RELEASE_POOL;
+
+      g_value_set_int (value, t * 1000);
       return TRUE;
     }
   
