@@ -26,6 +26,10 @@
 
 static gpointer parent_class;
 
+static GSList *update_windows = NULL;
+static guint update_idle = 0;
+
+
 NSView *
 gdk_quartz_window_get_nsview (GdkWindow *window)
 {
@@ -128,12 +132,12 @@ gdk_window_impl_quartz_class_init (GdkWindowImplQuartzClass *klass)
   drawable_class->get_visible_region = gdk_window_impl_quartz_get_visible_region;
 }
 
-
 static void
 gdk_window_impl_quartz_init (GdkWindowImplQuartz *impl)
 {
   impl->width = 1;
   impl->height = 1;
+  impl->type_hint = GDK_WINDOW_TYPE_HINT_NORMAL;
 }
 
 static void
@@ -141,9 +145,12 @@ gdk_window_impl_quartz_begin_paint_region (GdkPaintable *paintable,
 					   GdkRegion    *region)
 {
   GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (paintable);
-  CGContextRef context = _gdk_quartz_drawable_get_context (GDK_DRAWABLE (impl), FALSE);
-  int i, n_rects;
+  int n_rects;
   GdkRectangle *rects;
+  GdkPixmap *bg_pixmap;
+  GdkWindow *window;
+  
+  bg_pixmap = GDK_WINDOW_OBJECT (GDK_DRAWABLE_IMPL_QUARTZ (impl)->wrapper)->bg_pixmap;
 
   if (impl->begin_paint_count == 0)
     impl->paint_clip_region = gdk_region_copy (region);
@@ -152,18 +159,76 @@ gdk_window_impl_quartz_begin_paint_region (GdkPaintable *paintable,
 
   impl->begin_paint_count ++;
 
+  if (bg_pixmap == GDK_NO_BG)
+    return;
+	
   gdk_region_get_rectangles (region, &rects, &n_rects);
-  for (i = 0; i < n_rects; i++) 
+  
+  if (bg_pixmap == NULL)
     {
-      _gdk_quartz_set_context_fill_color_from_pixel (context, gdk_drawable_get_colormap (GDK_DRAWABLE_IMPL_QUARTZ (impl)->wrapper),
-						     GDK_WINDOW_OBJECT (GDK_DRAWABLE_IMPL_QUARTZ (impl)->wrapper)->bg_color.pixel);
-      
-      CGContextFillRect (context, CGRectMake (rects[i].x, rects[i].y, rects[i].width, rects[i].height));
+      CGContextRef context = gdk_quartz_drawable_get_context (GDK_DRAWABLE (impl), FALSE);
+      gint i;
+    
+      for (i = 0; i < n_rects; i++) 
+        {
+          gdk_quartz_set_context_fill_color_from_pixel 
+	    (context, gdk_drawable_get_colormap (GDK_DRAWABLE_IMPL_QUARTZ (impl)->wrapper),
+	     GDK_WINDOW_OBJECT (GDK_DRAWABLE_IMPL_QUARTZ (impl)->wrapper)->bg_color.pixel);
+	  
+          CGContextFillRect (context, CGRectMake (rects[i].x, rects[i].y, rects[i].width, rects[i].height));
+        }
       
+      gdk_quartz_drawable_release_context (GDK_DRAWABLE (impl), context);
     }
-  g_free (rects);
+  else
+    {
+      int x, y;
+      int x_offset, y_offset;
+      int width, height;
+      GdkGC *gc;
+      
+      x_offset = y_offset = 0;
+      
+      window = GDK_WINDOW (GDK_DRAWABLE_IMPL_QUARTZ (impl));
+      while (window && ((GdkWindowObject *) window)->bg_pixmap == GDK_PARENT_RELATIVE_BG)
+        {
+          /* If this window should have the same background as the parent,
+           * fetch the parent. (And if the same goes for the parent, fetch
+           * the grandparent, etc.)
+           */
+          x_offset += ((GdkWindowObject *) window)->x;
+          y_offset += ((GdkWindowObject *) window)->y;
+          window = GDK_WINDOW (((GdkWindowObject *) window)->parent);
+        }
 
-  _gdk_quartz_drawable_release_context (GDK_DRAWABLE (impl), context);
+      /* Note: There should be a CG API to draw tiled images, we might
+       * want to look into that for this. 
+       */
+      gc = gdk_gc_new (GDK_DRAWABLE (impl));
+      
+      gdk_drawable_get_size (GDK_DRAWABLE (bg_pixmap), &width, &height);
+      
+      x = -x_offset;
+      while (x < (rects[0].x + rects[0].width))
+        {
+          if (x + width >= rects[0].x)
+	    {
+              y = -y_offset;
+              while (y < (rects[0].y + rects[0].height))
+                {
+                  if (y + height >= rects[0].y)
+                    gdk_draw_drawable (GDK_DRAWABLE (impl), gc, bg_pixmap, 0, 0, x, y, width, height);
+		  
+                  y += height;
+                }
+            }
+          x += width;
+        }
+      
+      g_object_unref (G_OBJECT (gc));
+    }
+  
+  g_free (rects);
 }
 
 static void
@@ -180,27 +245,88 @@ gdk_window_impl_quartz_end_paint (GdkPaintable *paintable)
     }
 }
 
+static void
+gdk_window_quartz_process_all_updates (void)
+{
+  GSList *old_update_windows = update_windows;
+  GSList *tmp_list = update_windows;
+
+  update_idle = 0;
+  update_windows = NULL;
+
+  g_slist_foreach (old_update_windows, (GFunc) g_object_ref, NULL);
+  
+  while (tmp_list)
+    {
+      GdkWindowObject *private = tmp_list->data;
+      GdkWindowImplQuartz *impl = (GdkWindowImplQuartz *) private->impl;
+      int i, n_rects;
+      GdkRectangle *rects;
+      
+      if (private->update_area)
+	{
+	  gdk_region_get_rectangles (private->update_area, &rects, &n_rects);
+
+	  gdk_region_destroy (private->update_area);
+	  private->update_area = NULL;
+	  
+	  for (i = 0; i < n_rects; i++) 
+	    {
+	      [impl->view setNeedsDisplayInRect:NSMakeRect (rects[i].x, rects[i].y,
+							    rects[i].width, rects[i].height)];
+	    }
+	  
+	  [impl->view displayIfNeeded];
+
+	  g_free (rects);
+	}
+
+      g_object_unref (tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+
+  g_slist_free (old_update_windows);
+}
+
+static gboolean
+gdk_window_quartz_update_idle (gpointer data)
+{
+  GDK_THREADS_ENTER ();
+  gdk_window_quartz_process_all_updates ();
+  GDK_THREADS_LEAVE ();
+
+  return FALSE;
+}
+
 static void
 gdk_window_impl_quartz_invalidate_maybe_recurse (GdkPaintable *paintable,
 						 GdkRegion    *region,
 						 gboolean    (*child_func) (GdkWindow *, gpointer),
 						 gpointer      user_data)
 {
-  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (paintable);
-  int i, n_rects;
-  GdkRectangle *rects;
+  GdkWindowImplQuartz *window_impl = GDK_WINDOW_IMPL_QUARTZ (paintable);
+  GdkDrawableImplQuartz *drawable_impl = (GdkDrawableImplQuartz *) window_impl;
+  GdkWindow *window = (GdkWindow *) drawable_impl->wrapper;
+  GdkWindowObject *private = (GdkWindowObject *) window;
+  GdkRegion *visible_region;
 
-  gdk_region_get_rectangles (region, &rects, &n_rects);
+  visible_region = gdk_drawable_get_visible_region (window);
+  gdk_region_intersect (visible_region, region);
 
-  for (i = 0; i < n_rects; i++) 
+  if (private->update_area)
     {
-      [impl->view setNeedsDisplayInRect:NSMakeRect (rects[i].x, rects[i].y,
-						    rects[i].width, rects[i].height)];
+      gdk_region_union (private->update_area, visible_region);
+      gdk_region_destroy (visible_region);
     }
+  else
+    {
+      update_windows = g_slist_prepend (update_windows, window);
+      private->update_area = visible_region;
 
-  g_free (rects);
-
-  /* FIXME: Check if we need to traverse the children */
+      if (update_idle == 0)
+	update_idle = g_idle_add_full (GDK_PRIORITY_REDRAW,
+				       gdk_window_quartz_update_idle, NULL, NULL);
+    }
 }
 
 static void
@@ -208,10 +334,17 @@ gdk_window_impl_quartz_process_updates (GdkPaintable *paintable,
 					gboolean      update_children)
 {
   GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (paintable);
+  GdkDrawableImplQuartz *drawable_impl = (GdkDrawableImplQuartz *) impl;
+  GdkWindowObject *private = (GdkWindowObject *) drawable_impl->wrapper;
 
-  [impl->view display];
+  if (private->update_area)
+    {
+      gdk_region_destroy (private->update_area);
+      private->update_area = NULL;
+    }  
 
-  /* FIXME: Check if display actually updates the children too */
+  [impl->view setNeedsDisplay: YES];
+  update_windows = g_slist_remove (update_windows, private);
 }
 
 static void
@@ -224,7 +357,6 @@ gdk_window_impl_quartz_paintable_init (GdkPaintableIface *iface)
   iface->process_updates = gdk_window_impl_quartz_process_updates;
 }
 
-
 GType
 _gdk_window_impl_quartz_get_type (void)
 {
@@ -289,6 +421,72 @@ _gdk_quartz_get_inverted_screen_y (gint y)
   return rect.size.height - y;
 }
 
+static GdkWindow *
+find_child_window_by_point_helper (GdkWindow *window,
+				   int        x,
+				   int        y,
+				   int        x_offset,
+				   int        y_offset,
+				   int       *x_ret,
+				   int       *y_ret)
+{
+  GList *l;
+
+  for (l = GDK_WINDOW_OBJECT (window)->children; l; l = l->next)
+    {
+      GdkWindowObject *private = GDK_WINDOW_OBJECT (l->data);
+      GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
+      int temp_x, temp_y;
+
+      if (!GDK_WINDOW_IS_MAPPED (private))
+	continue;
+
+      temp_x = x_offset + private->x;
+      temp_y = y_offset + private->y;
+      
+      /* FIXME: Are there off by one errors here? */
+      if (x >= temp_x && y >= temp_y &&
+	  x < temp_x + impl->width && y < temp_y + impl->height) 
+	{
+	  *x_ret = x - x_offset - private->x;
+	  *y_ret = y - y_offset - private->y;
+	  
+	  /* Look for child windows */
+	  return find_child_window_by_point_helper (GDK_WINDOW (l->data),
+						    x, y,
+						    temp_x, temp_y,
+						    x_ret, y_ret);
+	}
+    }
+
+  return window;
+}
+
+/* Given a toplevel window and coordinates, returns the window
+ * in which the point is. Note that x and y should be non-flipped
+ * (and relative the toplevel), while the returned positions are
+ * flipped.
+ */
+GdkWindow *
+_gdk_quartz_find_child_window_by_point (GdkWindow *toplevel,
+					int        x,
+					int        y,
+					int       *x_ret,
+					int       *y_ret)
+{
+  GdkWindowObject *private = (GdkWindowObject *)toplevel;
+  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
+
+  /* If the point is in the title bar, ignore it */
+  if (y > impl->height)
+    return NULL;
+
+  /* First flip the y coordinate */
+  y = impl->height - y;
+
+  return find_child_window_by_point_helper (toplevel, x, y, 0, 0, x_ret, y_ret);
+}
+
 GdkWindow *
 gdk_window_new (GdkWindow     *parent,
 		GdkWindowAttr *attributes,
@@ -300,13 +498,10 @@ gdk_window_new (GdkWindow     *parent,
   GdkDrawableImplQuartz *draw_impl;
   GdkVisual *visual;
 
-  GDK_QUARTZ_ALLOC_POOL;
-
   if (parent && GDK_WINDOW_DESTROYED (parent))
-    {
-      GDK_QUARTZ_RELEASE_POOL;
-      return NULL;
-    }
+    return NULL;
+
+  GDK_QUARTZ_ALLOC_POOL;
 
   if (!parent)
     parent = _gdk_root;
@@ -370,6 +565,7 @@ gdk_window_new (GdkWindow     *parent,
       break;
     default:
       g_warning (G_STRLOC "cannot make windows of type %d", private->window_type);
+      GDK_QUARTZ_RELEASE_POOL;
       return NULL;
     }
 
@@ -436,22 +632,23 @@ gdk_window_new (GdkWindow     *parent,
 	  style_mask = NSBorderlessWindowMask;
 	  break;
 	default:
-	  style_mask = NSTitledWindowMask|NSClosableWindowMask|NSMiniaturizableWindowMask|NSResizableWindowMask;
+	  style_mask = (NSTitledWindowMask |
+			NSClosableWindowMask |
+			NSMiniaturizableWindowMask |
+			NSResizableWindowMask);
 	} 
 
 	impl->toplevel = [[GdkQuartzWindow alloc] initWithContentRect:content_rect 
 			                                    styleMask:style_mask
-			                                      backing:NSBackingStoreBuffered defer:NO];
+			                                      backing:NSBackingStoreBuffered
+			                                        defer:NO];
+
 	if (attributes_mask & GDK_WA_TITLE)
 	  title = attributes->title;
 	else
 	  title = get_default_title ();
 
-	if (attributes->window_type == GDK_WINDOW_TEMP)
-	  [impl->toplevel setLevel:NSPopUpMenuWindowLevel];
-
 	gdk_window_set_title (window, title);
-
 	  
 	if (draw_impl->colormap == gdk_screen_get_rgba_colormap (_gdk_screen))
 	  {
@@ -494,6 +691,9 @@ gdk_window_new (GdkWindow     *parent,
 
   GDK_QUARTZ_RELEASE_POOL;
 
+  if (attributes_mask & GDK_WA_TYPE_HINT)
+    gdk_window_set_type_hint (window, attributes->type_hint);
+
   return window;
 }
 
@@ -518,7 +718,9 @@ _gdk_windowing_window_destroy (GdkWindow *window,
 			       gboolean   recursing,
 			       gboolean   foreign_destroy)
 {
-  if (!recursing && ! foreign_destroy)
+  update_windows = g_slist_remove (update_windows, window);
+
+  if (!recursing && !foreign_destroy)
     {
       GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
 
@@ -531,14 +733,13 @@ _gdk_windowing_window_destroy (GdkWindow *window,
 	[impl->toplevel close];
       else if (impl->view)
 	[impl->view release];
-      
     }
 }
 
 void
 _gdk_windowing_window_destroy_foreign (GdkWindow *window)
 {
-  /* FIXME: Implement */
+  /* Foreign windows aren't supported in OSX. */
 }
 
 static gboolean
@@ -556,22 +757,20 @@ all_parents_shown (GdkWindowObject *private)
 }
 
 static void
-show_window_internal (GdkWindow *window, gboolean raise)
+show_window_internal (GdkWindow *window,
+		      gboolean   raise)
 {
   GdkWindowObject *private;
   GdkWindowImplQuartz *impl;
 
-  private = (GdkWindowObject *)window;
-
-  if (private->destroyed)
+  if (GDK_WINDOW_DESTROYED (window))
     return;
 
   GDK_QUARTZ_ALLOC_POOL;
 
+  private = (GdkWindowObject *)window;
   impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
 
-  /* FIXME: We need to raise the window (move it to the top in the list) */
-
   if (impl->toplevel)
     {
       [impl->toplevel orderFront:nil];
@@ -588,6 +787,12 @@ show_window_internal (GdkWindow *window, gboolean raise)
 
   gdk_synthesize_window_state (window, GDK_WINDOW_STATE_WITHDRAWN, 0);
 
+  if (private->state & GDK_WINDOW_STATE_MAXIMIZED)
+    gdk_window_maximize (window);
+
+  if (private->state & GDK_WINDOW_STATE_ICONIFIED)
+    gdk_window_iconify (window);
+
   GDK_QUARTZ_RELEASE_POOL;
 }
 
@@ -676,6 +881,8 @@ move_resize_window_internal (GdkWindow *window,
   if (height != -1)
     impl->height = height;
 
+  GDK_QUARTZ_ALLOC_POOL;
+
   if (impl->toplevel)
     {
       NSRect content_rect = NSMakeRect (private->x, 
@@ -693,11 +900,13 @@ move_resize_window_internal (GdkWindow *window,
 	{
 	  [impl->view setFrame:NSMakeRect (private->x, private->y, 
 					   impl->width, impl->height)];
-	  
+
 	  /* FIXME: Maybe we should use setNeedsDisplayInRect instead */
 	  [impl->view setNeedsDisplay:YES];
 	}
     }
+
+  GDK_QUARTZ_RELEASE_POOL;
 }
 
 void
@@ -817,11 +1026,37 @@ gdk_window_set_back_pixmap (GdkWindow *window,
 			    GdkPixmap *pixmap,
 			    gboolean   parent_relative)
 {
+  GdkWindowObject *private = (GdkWindowObject *)window;
+
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (pixmap == NULL || !parent_relative);
   g_return_if_fail (pixmap == NULL || gdk_drawable_get_depth (window) == gdk_drawable_get_depth (pixmap));
 
-  /* FIXME: Implement */
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (private->bg_pixmap &&
+      private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
+      private->bg_pixmap != GDK_NO_BG)
+    g_object_unref (private->bg_pixmap);
+
+  if (parent_relative)
+    {
+      private->bg_pixmap = GDK_PARENT_RELATIVE_BG;
+      GDK_NOTE (MISC, g_print (G_STRLOC ": setting background pixmap to parent_relative\n"));
+    }
+  else
+    {
+      if (pixmap)
+	{
+	  g_object_ref (pixmap);
+	  private->bg_pixmap = pixmap;
+	}
+      else
+	{
+	  private->bg_pixmap = GDK_NO_BG;
+	}
+    }
 }
 
 void
@@ -922,8 +1157,7 @@ gdk_window_get_deskrelative_origin (GdkWindow *window,
 {
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
 
-  /* FIXME: Implement */
-  return FALSE;
+  return gdk_window_get_origin (window, x, y);
 }
 
 void
@@ -931,9 +1165,20 @@ gdk_window_get_root_origin (GdkWindow *window,
 			    gint      *x,
 			    gint      *y)
 {
+  GdkRectangle rect;
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  /* FIXME: Implement */
+  rect.x = 0;
+  rect.y = 0;
+  
+  gdk_window_get_frame_extents (window, &rect);
+
+  if (x)
+    *x = rect.x;
+
+  if (y)
+    *y = rect.y;
 }
 
 void
@@ -943,8 +1188,13 @@ _gdk_windowing_get_pointer (GdkDisplay       *display,
 			    gint             *y,
 			    GdkModifierType  *mask)
 {
+  g_return_if_fail (display == _gdk_display);
+  
+  *screen = _gdk_screen;
+  _gdk_windowing_window_get_pointer (_gdk_display, _gdk_root, x, y, mask);
 }
 
+/* Returns coordinates relative to the upper left corner of window. */
 GdkWindow *
 _gdk_windowing_window_get_pointer (GdkDisplay      *display,
 				   GdkWindow       *window,
@@ -952,33 +1202,54 @@ _gdk_windowing_window_get_pointer (GdkDisplay      *display,
 				   gint            *y,
 				   GdkModifierType *mask)
 {
-  GdkWindow *toplevel = gdk_window_get_toplevel (window);
-  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (toplevel)->impl);
-  GdkWindowObject *private = GDK_WINDOW_OBJECT (window);
-  NSWindow *nswindow = impl->toplevel;
-  NSPoint point = [nswindow mouseLocationOutsideOfEventStream];
-  int x_tmp, y_tmp;
+  GdkWindow *toplevel;
+  GdkWindowImplQuartz *impl;
+  GdkWindowObject *private;
+  NSPoint point;
+  gint x_tmp, y_tmp;
 
-  /* FIXME: Might need to special-case window being the root window. */
+  if (GDK_WINDOW_DESTROYED (window))
+    {
+      *x = 0;
+      *y = 0;
+      *mask = 0;
+      return NULL;
+    }
   
-  /* First flip the y coordinate */
+  toplevel = gdk_window_get_toplevel (window);
+  impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (toplevel)->impl);
+  private = GDK_WINDOW_OBJECT (window);
+
+  /* Must flip the y coordinate. */
+  if (window == _gdk_root)
+    {
+      point = [NSEvent mouseLocation];
+      y_tmp = _gdk_quartz_get_inverted_screen_y (point.y);
+      *mask = _gdk_quartz_get_current_event_mask ();
+    }
+  else
+    {
+      NSWindow *nswindow = impl->toplevel;
+      point = [nswindow mouseLocationOutsideOfEventStream];
+      y_tmp = impl->height - point.y;
+      *mask = _gdk_quartz_get_current_event_mask ();
+    }
   x_tmp = point.x;
-  y_tmp = impl->height - point.y;
-      
-  while (private != GDK_WINDOW_OBJECT (toplevel)) {
-    x_tmp -= private->x;
-    y_tmp -= private->y;
+
+  while (private != GDK_WINDOW_OBJECT (toplevel))
+    {
+      x_tmp -= private->x;
+      y_tmp -= private->y;
     
-    private = private->parent;
-  }
+      private = private->parent;
+    }
 
-  if (x)
-    *x = x_tmp;
-  if (y)
-    *y = y_tmp;
+  *x = x_tmp;
+  *y = y_tmp;
 
-  /* FIXME: Implement return value */
-  return NULL;
+  return _gdk_quartz_find_child_window_by_point (window,
+						 point.x, point.y,
+						 &x_tmp, &y_tmp);
 }
 
 void
@@ -995,8 +1266,12 @@ _gdk_windowing_window_at_pointer (GdkDisplay *display,
 				  gint       *win_x,
 				  gint       *win_y)
 {
-  /* FIXME: Implement */
-  return NULL;
+  GdkModifierType mask;
+
+  return _gdk_windowing_window_get_pointer (display,
+					    _gdk_root,
+					    win_x, win_y,
+					    &mask);
 }
 
 GdkEventMask  
@@ -1034,7 +1309,77 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
 			       GdkGeometry    *geometry,
 			       GdkWindowHints  geom_mask)
 {
-  /* FIXME: Implement */
+  GdkWindowImplQuartz *impl;
+
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_if_fail (geometry != NULL);
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+  
+  impl = GDK_WINDOW_IMPL_QUARTZ (((GdkWindowObject *) window)->impl);
+  if (!impl->toplevel)
+    return;
+
+  if (geom_mask & GDK_HINT_POS)
+    {
+      /* FIXME: Implement */
+    }
+
+  if (geom_mask & GDK_HINT_USER_POS)
+    {
+      /* FIXME: Implement */
+    }
+
+  if (geom_mask & GDK_HINT_USER_SIZE)
+    {
+      /* FIXME: Implement */
+    }
+  
+  if (geom_mask & GDK_HINT_MIN_SIZE)
+    {
+      NSSize size;
+
+      size.width = geometry->min_width;
+      size.height = geometry->min_height;
+
+      [impl->toplevel setContentMinSize:size];
+    }
+  
+  if (geom_mask & GDK_HINT_MAX_SIZE)
+    {
+      NSSize size;
+
+      size.width = geometry->max_width;
+      size.height = geometry->max_height;
+
+      [impl->toplevel setContentMaxSize:size];
+    }
+  
+  if (geom_mask & GDK_HINT_BASE_SIZE)
+    {
+      /* FIXME: Implement */
+    }
+  
+  if (geom_mask & GDK_HINT_RESIZE_INC)
+    {
+      NSSize size;
+
+      size.width = geometry->width_inc;
+      size.height = geometry->height_inc;
+
+      [impl->toplevel setContentResizeIncrements:size];
+    }
+  
+  if (geom_mask & GDK_HINT_ASPECT)
+    {
+      /* FIXME: Implement */
+    }
+
+  if (geom_mask & GDK_HINT_WIN_GRAVITY)
+    {
+      /* FIXME: Implement */
+    }
 }
 
 void
@@ -1053,7 +1398,9 @@ gdk_window_set_title (GdkWindow   *window,
 
   if (impl->toplevel)
     {
+      GDK_QUARTZ_ALLOC_POOL;
       [impl->toplevel setTitle:[NSString stringWithUTF8String:title]];
+      GDK_QUARTZ_RELEASE_POOL;
     }
 }
 
@@ -1064,7 +1411,7 @@ gdk_window_set_role (GdkWindow   *window,
   /* FIXME: Implement */
 }
 
-void          
+void
 gdk_window_set_transient_for (GdkWindow *window, 
 			      GdkWindow *parent)
 {
@@ -1171,7 +1518,6 @@ gdk_window_set_icon (GdkWindow *window,
 		     GdkPixmap *pixmap,
 		     GdkBitmap *mask)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   /* FIXME: Implement */
@@ -1181,7 +1527,6 @@ void
 gdk_window_set_icon_name (GdkWindow   *window, 
 			  const gchar *name)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   /* FIXME: Implement */
@@ -1213,9 +1558,92 @@ void
 gdk_window_set_type_hint (GdkWindow        *window,
 			  GdkWindowTypeHint hint)
 {
+  GdkWindowImplQuartz *impl;
+  gint                 level;
+  gboolean             shadow;
+  
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  /* FIXME: Implement */
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  impl = GDK_WINDOW_IMPL_QUARTZ (((GdkWindowObject *) window)->impl);
+
+  impl->type_hint = hint;
+
+  /* Match the documentation, only do something if we're not mapped yet. */
+  if (GDK_WINDOW_IS_MAPPED (window))
+    return;
+
+  switch (hint)
+    {
+    case GDK_WINDOW_TYPE_HINT_NORMAL:  /* Normal toplevel window */
+    case GDK_WINDOW_TYPE_HINT_DIALOG:  /* Dialog window */
+      level = NSNormalWindowLevel;
+      shadow = TRUE;
+      break;
+
+    case GDK_WINDOW_TYPE_HINT_TOOLBAR: /* Window used to implement toolbars */
+    case GDK_WINDOW_TYPE_HINT_DESKTOP: /* N/A */
+      level = NSNormalWindowLevel;
+      shadow = FALSE;
+      break;
+
+    case GDK_WINDOW_TYPE_HINT_DOCK:
+    case GDK_WINDOW_TYPE_HINT_UTILITY:
+      level = NSFloatingWindowLevel;
+      shadow = TRUE;
+      break;
+
+    case GDK_WINDOW_TYPE_HINT_MENU: /* Torn-off menu */
+      level = NSTornOffMenuWindowLevel;
+      shadow = TRUE;
+      break;
+      
+    case GDK_WINDOW_TYPE_HINT_DROPDOWN_MENU: /* Menu from menubar */
+      level = NSTornOffMenuWindowLevel;
+      shadow = TRUE;
+      break;
+
+    case GDK_WINDOW_TYPE_HINT_SPLASHSCREEN:
+      level = NSPopUpMenuWindowLevel;
+      shadow = TRUE;
+      break;
+
+    case GDK_WINDOW_TYPE_HINT_POPUP_MENU:
+    case GDK_WINDOW_TYPE_HINT_COMBO:
+      level = NSPopUpMenuWindowLevel;
+      shadow = TRUE;
+      break;
+      
+    case GDK_WINDOW_TYPE_HINT_NOTIFICATION:
+    case GDK_WINDOW_TYPE_HINT_TOOLTIP:
+      level = NSStatusWindowLevel;
+      shadow = TRUE;
+      break;
+
+    case GDK_WINDOW_TYPE_HINT_DND:
+      level = NSPopUpMenuWindowLevel;
+      shadow = FALSE;
+      break;
+
+    default:
+      level = NSNormalWindowLevel;
+      shadow = FALSE;
+      break;
+    }
+
+  [impl->toplevel setHasShadow:shadow];
+  [impl->toplevel setLevel:level];
+}
+
+GdkWindowTypeHint
+gdk_window_get_type_hint (GdkWindow *window)
+{
+  if (GDK_WINDOW_DESTROYED (window))
+    return GDK_WINDOW_TYPE_HINT_NORMAL;
+  
+  return GDK_WINDOW_IMPL_QUARTZ (((GdkWindowObject *) window)->impl)->type_hint;
 }
 
 void
@@ -1281,7 +1709,33 @@ void
 gdk_window_get_frame_extents (GdkWindow    *window,
                               GdkRectangle *rect)
 {
-  /* FIXME: Implement */
+  GdkWindowObject *private;
+  GdkWindow *toplevel;
+  GdkWindowImplQuartz *impl;
+  NSRect ns_rect;
+
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_if_fail (rect != NULL);
+
+  private = GDK_WINDOW_OBJECT (window);
+
+  rect->x = 0;
+  rect->y = 0;
+  rect->width = 1;
+  rect->height = 1;
+  
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  toplevel = gdk_window_get_toplevel (window);
+  impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (toplevel)->impl);
+
+  ns_rect = [impl->toplevel frame];
+
+  rect->x = ns_rect.origin.x;
+  rect->y = _gdk_quartz_get_inverted_screen_y (ns_rect.origin.y + ns_rect.size.height);
+  rect->width = ns_rect.size.width;
+  rect->height = ns_rect.size.height;
 }
 
 void
@@ -1294,8 +1748,8 @@ gdk_window_set_decorations (GdkWindow      *window,
 }
 
 gboolean
-gdk_window_get_decorations(GdkWindow       *window,
-			   GdkWMDecoration *decorations)
+gdk_window_get_decorations (GdkWindow       *window,
+			    GdkWMDecoration *decorations)
 {
   /* FIXME: Implement */
   return FALSE;
@@ -1340,44 +1794,94 @@ gdk_window_unstick (GdkWindow *window)
 void
 gdk_window_maximize (GdkWindow *window)
 {
+  GdkWindowImplQuartz *impl;
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  /* FIXME: Implement */
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {
+      GDK_QUARTZ_ALLOC_POOL;
+
+      if (impl->toplevel && ![impl->toplevel isZoomed])
+	[impl->toplevel zoom:nil];
+
+      GDK_QUARTZ_RELEASE_POOL;
+    }
+  else
+    {
+      gdk_synthesize_window_state (window,
+				   0,
+				   GDK_WINDOW_STATE_MAXIMIZED);
+    }
 }
 
 void
 gdk_window_unmaximize (GdkWindow *window)
 {
+  GdkWindowImplQuartz *impl;
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  /* FIXME: Implement */
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
+
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {
+      GDK_QUARTZ_ALLOC_POOL;
+
+      if (impl->toplevel && [impl->toplevel isZoomed])
+	[impl->toplevel zoom:nil];
+
+      GDK_QUARTZ_RELEASE_POOL;
+    }
+  else
+    {
+      gdk_synthesize_window_state (window,
+				   GDK_WINDOW_STATE_MAXIMIZED,
+				   0);
+    }
 }
 
 void
 gdk_window_iconify (GdkWindow *window)
 {
   GdkWindowImplQuartz *impl;
-  NSAutoreleasePool *pool;
 
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (GDK_WINDOW_DESTROYED (window))
     return;
-
+  
   impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
-  if (!impl->toplevel)
-    return;
 
-  pool = [[NSAutoreleasePool alloc] init];
-  [impl->toplevel miniaturize:nil];
-  [pool release];
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {
+      GDK_QUARTZ_ALLOC_POOL;
+
+      if (impl->toplevel)
+	[impl->toplevel miniaturize:nil];
+
+      GDK_QUARTZ_RELEASE_POOL;
+    }
+  else
+    {
+      gdk_synthesize_window_state (window,
+				   0,
+				   GDK_WINDOW_STATE_ICONIFIED);
+    }
 }
 
 void
 gdk_window_deiconify (GdkWindow *window)
 {
   GdkWindowImplQuartz *impl;
-  NSAutoreleasePool *pool;
 
   g_return_if_fail (GDK_IS_WINDOW (window));
   
@@ -1385,12 +1889,22 @@ gdk_window_deiconify (GdkWindow *window)
     return;
 
   impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
-  if (!impl->toplevel)
-    return;
 
-  pool = [[NSAutoreleasePool alloc] init];
-  [impl->toplevel deminiaturize:nil];
-  [pool release];
+  if (GDK_WINDOW_IS_MAPPED (window))
+    {
+      GDK_QUARTZ_ALLOC_POOL;
+
+      if (impl->toplevel)
+	[impl->toplevel deminiaturize:nil];
+
+      GDK_QUARTZ_RELEASE_POOL;
+    }
+  else
+    {
+      gdk_synthesize_window_state (window,
+				   GDK_WINDOW_STATE_ICONIFIED,
+				   0);
+    }
 }
 
 void
@@ -1474,3 +1988,11 @@ void
 gdk_window_configure_finished (GdkWindow *window)
 {
 }
+
+void
+gdk_window_destroy_notify (GdkWindow *window)
+{
+  /* FIXME: Implement. We should call this from -[GdkQuartzWindow dealloc] or
+   * -[GdkQuartzView dealloc], although I suspect that currently they leak
+   * anyway. */
+}
