@@ -361,13 +361,15 @@ get_default_title (void)
  *   get its own class
  */
 static ATOM
-RegisterGdkClass (GdkWindowType wtype)
+RegisterGdkClass (GdkWindowType wtype, GdkWindowTypeHint wtype_hint)
 {
-  static ATOM klassTOPLEVEL = 0;
-  static ATOM klassDIALOG   = 0;
-  static ATOM klassCHILD    = 0;
-  static ATOM klassTEMP     = 0;
+  static ATOM klassTOPLEVEL   = 0;
+  static ATOM klassDIALOG     = 0;
+  static ATOM klassCHILD      = 0;
+  static ATOM klassTEMP       = 0;
+  static ATOM klassTEMPSHADOW = 0;
   static HICON hAppIcon = NULL;
+  static HICON hAppIconSm = NULL;
   static WNDCLASSEX wcl; 
   ATOM klass = 0;
 
@@ -380,26 +382,37 @@ RegisterGdkClass (GdkWindowType wtype)
   wcl.cbWndExtra = 0;
   wcl.hInstance = _gdk_app_hmodule;
   wcl.hIcon = 0;
+  wcl.hIconSm = 0;
   /* initialize once! */
-  if (0 == hAppIcon)
+  if (0 == hAppIcon && 0 == hAppIconSm)
     {
       gchar sLoc [MAX_PATH+1];
 
       if (0 != GetModuleFileName (_gdk_app_hmodule, sLoc, MAX_PATH))
-	{
-	  hAppIcon = ExtractIcon (_gdk_app_hmodule, sLoc, 0);
-	  if (0 == hAppIcon)
-	    {
-	      if (0 != GetModuleFileName (_gdk_dll_hinstance, sLoc, MAX_PATH))
-		hAppIcon = ExtractIcon (_gdk_dll_hinstance, sLoc, 0);
-	    }
-	}
-      if (0 == hAppIcon) 
-	hAppIcon = LoadIcon (NULL, IDI_APPLICATION);
+        {
+          ExtractIconEx (sLoc, 0, &hAppIcon, &hAppIconSm, 1);
+          if (0 == hAppIcon && 0 == hAppIconSm)
+            {
+              if (0 != GetModuleFileName (_gdk_dll_hinstance, sLoc, MAX_PATH))
+                ExtractIconEx (sLoc, 0, &hAppIcon, &hAppIconSm, 1);
+            }
+        }
+      if (0 == hAppIcon && 0 == hAppIconSm)
+        {
+          hAppIcon = LoadImage (NULL, IDI_APPLICATION, IMAGE_ICON,
+                                GetSystemMetrics (SM_CXICON),
+                                GetSystemMetrics (SM_CYICON), 0);
+          hAppIconSm = LoadImage (NULL, IDI_APPLICATION, IMAGE_ICON,
+                                  GetSystemMetrics (SM_CXSMICON),
+                                  GetSystemMetrics (SM_CYSMICON), 0);
+        }
     }
+  if (0 == hAppIcon)
+    hAppIcon = hAppIconSm;
+  else if (0 == hAppIconSm)
+    hAppIconSm = hAppIcon;
 
   wcl.lpszMenuName = NULL;
-  wcl.hIconSm = 0;
 
   /* initialize once per class */
   /*
@@ -409,7 +422,7 @@ RegisterGdkClass (GdkWindowType wtype)
    */
 #define ONCE_PER_CLASS() \
   wcl.hIcon = CopyIcon (hAppIcon); \
-  wcl.hIconSm = CopyIcon (hAppIcon); \
+  wcl.hIconSm = CopyIcon (hAppIconSm); \
   wcl.hbrBackground = NULL; \
   wcl.hCursor = LoadCursor (NULL, IDC_ARROW); 
   
@@ -450,14 +463,33 @@ RegisterGdkClass (GdkWindowType wtype)
       break;
       
     case GDK_WINDOW_TEMP:
-      if (0 == klassTEMP)
+      if ((wtype_hint == GDK_WINDOW_TYPE_HINT_MENU) ||
+	  (wtype_hint == GDK_WINDOW_TYPE_HINT_DROPDOWN_MENU) ||
+	  (wtype_hint == GDK_WINDOW_TYPE_HINT_POPUP_MENU) ||
+	  (wtype_hint == GDK_WINDOW_TYPE_HINT_TOOLTIP))
 	{
-	  wcl.lpszClassName = "gdkWindowTemp";
-	  wcl.style |= CS_SAVEBITS;
-	  ONCE_PER_CLASS ();
-	  klassTEMP = RegisterClassEx (&wcl);
+	  if (klassTEMPSHADOW == 0)
+	    {
+	      wcl.lpszClassName = "gdkWindowTempShadow";
+	      wcl.style |= CS_SAVEBITS;
+	      if (_winver >= 0x0501) /* Windows XP (5.1) or above */
+		wcl.style |= 0x00020000; /* CS_DROPSHADOW */
+	      ONCE_PER_CLASS ();
+	      klassTEMPSHADOW = RegisterClassEx (&wcl);
+	    }
+	  klass = klassTEMPSHADOW;
+	}
+      else
+	{
+	  if (0 == klassTEMP)
+	    {
+	      wcl.lpszClassName = "gdkWindowTemp";
+	      wcl.style |= CS_SAVEBITS;
+	      ONCE_PER_CLASS ();
+	      klassTEMP = RegisterClassEx (&wcl);
+	    }
+	  klass = klassTEMP;
 	}
-      klass = klassTEMP;
       break;
       
     default:
@@ -672,10 +704,15 @@ gdk_window_new_internal (GdkWindow     *parent,
 
   private->event_mask = GDK_STRUCTURE_MASK | attributes->event_mask;
       
+  if (attributes_mask & GDK_WA_TYPE_HINT)
+    impl->type_hint = attributes->type_hint;
+  else
+    impl->type_hint = GDK_WINDOW_TYPE_HINT_NORMAL;
+
   if (private->parent)
     private->parent->children = g_list_prepend (private->parent->children, window);
 
-  klass = RegisterGdkClass (private->window_type);
+  klass = RegisterGdkClass (private->window_type, impl->type_hint);
 
   mbtitle = g_locale_from_utf8 (title, -1, NULL, NULL, NULL);
   
@@ -1409,6 +1446,139 @@ gdk_window_reparent (GdkWindow *window,
   _gdk_window_init_position (GDK_WINDOW (window_private));
 }
 
+static void
+erase_background (GdkWindow *window,
+		  HDC        hdc)
+{
+  HDC bgdc = NULL;
+  HBRUSH hbr = NULL;
+  HPALETTE holdpal = NULL;
+  RECT rect;
+  COLORREF bg;
+  GdkColormap *colormap;
+  GdkColormapPrivateWin32 *colormap_private;
+  int x, y;
+  int x_offset, y_offset;
+  
+  if (((GdkWindowObject *) window)->input_only ||
+      ((GdkWindowObject *) window)->bg_pixmap == GDK_NO_BG ||
+      GDK_WINDOW_IMPL_WIN32 (((GdkWindowObject *) window)->impl)->position_info.no_bg)
+    {
+      return;
+    }
+
+  colormap = gdk_drawable_get_colormap (window);
+
+  if (colormap &&
+      (colormap->visual->type == GDK_VISUAL_PSEUDO_COLOR ||
+       colormap->visual->type == GDK_VISUAL_STATIC_COLOR))
+    {
+      int k;
+	  
+      colormap_private = GDK_WIN32_COLORMAP_DATA (colormap);
+
+      if (!(holdpal = SelectPalette (hdc,  colormap_private->hpal, FALSE)))
+        WIN32_GDI_FAILED ("SelectPalette");
+      else if ((k = RealizePalette (hdc)) == GDI_ERROR)
+	WIN32_GDI_FAILED ("RealizePalette");
+      else if (k > 0)
+	GDK_NOTE (COLORMAP, g_print ("erase_background: realized %p: %d colors\n",
+				     colormap_private->hpal, k));
+    }
+  
+  x_offset = y_offset = 0;
+  while (window && ((GdkWindowObject *) window)->bg_pixmap == GDK_PARENT_RELATIVE_BG)
+    {
+      /* If this window should have the same background as the parent,
+       * fetch the parent. (And if the same goes for the parent, fetch
+       * the grandparent, etc.)
+       */
+      x_offset += ((GdkWindowObject *) window)->x;
+      y_offset += ((GdkWindowObject *) window)->y;
+      window = GDK_WINDOW (((GdkWindowObject *) window)->parent);
+    }
+  
+  if (GDK_WINDOW_IMPL_WIN32 (((GdkWindowObject *) window)->impl)->position_info.no_bg)
+    {
+      /* Improves scolling effect, e.g. main buttons of testgtk */
+      return;
+    }
+
+  GetClipBox (hdc, &rect);
+
+  if (((GdkWindowObject *) window)->bg_pixmap == NULL)
+    {
+      bg = _gdk_win32_colormap_color (GDK_DRAWABLE_IMPL_WIN32 (((GdkWindowObject *) window)->impl)->colormap,
+				      ((GdkWindowObject *) window)->bg_color.pixel);
+      
+      if (!(hbr = CreateSolidBrush (bg)))
+	WIN32_GDI_FAILED ("CreateSolidBrush");
+      else if (!FillRect (hdc, &rect, hbr))
+	WIN32_GDI_FAILED ("FillRect");
+      if (hbr != NULL)
+	DeleteObject (hbr);
+    }
+  else if (((GdkWindowObject *) window)->bg_pixmap != GDK_NO_BG)
+    {
+      GdkPixmap *pixmap = ((GdkWindowObject *) window)->bg_pixmap;
+      GdkPixmapImplWin32 *pixmap_impl = GDK_PIXMAP_IMPL_WIN32 (GDK_PIXMAP_OBJECT (pixmap)->impl);
+      
+      if (x_offset == 0 && y_offset == 0 &&
+	  pixmap_impl->width <= 8 && pixmap_impl->height <= 8)
+	{
+	  if (!(hbr = CreatePatternBrush (GDK_PIXMAP_HBITMAP (pixmap))))
+	    WIN32_GDI_FAILED ("CreatePatternBrush");
+	  else if (!FillRect (hdc, &rect, hbr))
+	    WIN32_GDI_FAILED ("FillRect");
+	  if (hbr != NULL)
+	    DeleteObject (hbr);
+	}
+      else
+	{
+	  HGDIOBJ oldbitmap;
+
+	  if (!(bgdc = CreateCompatibleDC (hdc)))
+	    {
+	      WIN32_GDI_FAILED ("CreateCompatibleDC");
+	      return;
+	    }
+	  if (!(oldbitmap = SelectObject (bgdc, GDK_PIXMAP_HBITMAP (pixmap))))
+	    {
+	      WIN32_GDI_FAILED ("SelectObject");
+	      DeleteDC (bgdc);
+	      return;
+	    }
+	  x = -x_offset;
+	  while (x < rect.right)
+	    {
+	      if (x + pixmap_impl->width >= rect.left)
+		{
+		  y = -y_offset;
+		  while (y < rect.bottom)
+		    {
+		      if (y + pixmap_impl->height >= rect.top)
+			{
+			  if (!BitBlt (hdc, x, y,
+				       pixmap_impl->width, pixmap_impl->height,
+				       bgdc, 0, 0, SRCCOPY))
+			    {
+			      WIN32_GDI_FAILED ("BitBlt");
+			      SelectObject (bgdc, oldbitmap);
+			      DeleteDC (bgdc);
+			      return;
+			    }
+			}
+		      y += pixmap_impl->height;
+		    }
+		}
+	      x += pixmap_impl->width;
+	    }
+	  SelectObject (bgdc, oldbitmap);
+	  DeleteDC (bgdc);
+	}
+    }
+}
+
 void
 _gdk_windowing_window_clear_area (GdkWindow *window,
 				  gint       x,
@@ -1435,8 +1605,9 @@ _gdk_windowing_window_clear_area (GdkWindow *window,
 			       GDK_WINDOW_HWND (window),
 			       width, height, x, y));
       hdc = GetDC (GDK_WINDOW_HWND (window));
-      IntersectClipRect (hdc, x, y, x + width + 1, y + height + 1);
-      SendMessage (GDK_WINDOW_HWND (window), WM_ERASEBKGND, (WPARAM) hdc, 0);
+      IntersectClipRect (hdc, x, y, x + width, y + height);
+      erase_background (window, hdc);
+
       GDI_CALL (ReleaseDC, (GDK_WINDOW_HWND (window), hdc));
     }
 }
@@ -1452,6 +1623,7 @@ _gdk_windowing_window_clear_area_e (GdkWindow *window,
   
   if (!GDK_WINDOW_DESTROYED (window))
     {
+      HDC hdc;
       RECT rect;
 
       GDK_NOTE (MISC, g_print ("_gdk_windowing_window_clear_area_e: %p: "
@@ -1459,10 +1631,18 @@ _gdk_windowing_window_clear_area_e (GdkWindow *window,
 			       GDK_WINDOW_HWND (window),
 			       width, height, x, y));
 
+      /* The background should be erased before the expose event is
+	 generated */
+      hdc = GetDC (GDK_WINDOW_HWND (window));
+      IntersectClipRect (hdc, x, y, x + width, y + height);
+      erase_background (window, hdc);
+      GDI_CALL (ReleaseDC, (GDK_WINDOW_HWND (window), hdc));
+
       rect.left = x;
-      rect.right = x + width + 1;
+      rect.right = x + width;
       rect.top = y;
-      rect.bottom = y + height + 1;
+      rect.bottom = y + height;
+
       GDI_CALL (InvalidateRect, (GDK_WINDOW_HWND (window), &rect, TRUE));
       UpdateWindow (GDK_WINDOW_HWND (window));
     }
@@ -2325,6 +2505,30 @@ gdk_window_set_events (GdkWindow   *window,
   GDK_WINDOW_OBJECT (window)->event_mask = GDK_STRUCTURE_MASK | event_mask;
 }
 
+static void
+do_shape_combine_region (GdkWindow *window,
+			 HRGN	    hrgn,
+			 gint       x, gint y)
+{
+  RECT rect;
+
+  GetClientRect (GDK_WINDOW_HWND (window), &rect);
+  _gdk_win32_adjust_client_rect (window, &rect);
+
+  OffsetRgn (hrgn, -rect.left, -rect.top);
+  OffsetRgn (hrgn, x, y);
+
+  /* If this is a top-level window, add the title bar to the region */
+  if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_TOPLEVEL)
+    {
+      HRGN tmp = CreateRectRgn (0, 0, rect.right - rect.left, -rect.top);
+      CombineRgn (hrgn, hrgn, tmp, RGN_OR);
+      DeleteObject (tmp);
+    }
+  
+  SetWindowRgn (GDK_WINDOW_HWND (window), hrgn, TRUE);
+}
+
 void
 gdk_window_shape_combine_mask (GdkWindow *window,
 			       GdkBitmap *mask,
@@ -2345,35 +2549,41 @@ gdk_window_shape_combine_mask (GdkWindow *window,
   else
     {
       HRGN hrgn;
-      RECT rect;
-
-      /* Convert mask bitmap to region */
-      hrgn = _gdk_win32_bitmap_to_hrgn (mask);
 
       GDK_NOTE (MISC, g_print ("gdk_window_shape_combine_mask: %p: %p\n",
 			       GDK_WINDOW_HWND (window),
 			       GDK_WINDOW_HWND (mask)));
 
-      GetClientRect (GDK_WINDOW_HWND (window), &rect);
-      _gdk_win32_adjust_client_rect (window, &rect);
+      /* Convert mask bitmap to region */
+      hrgn = _gdk_win32_bitmap_to_hrgn (mask);
 
-      OffsetRgn (hrgn, -rect.left, -rect.top);
-      OffsetRgn (hrgn, x, y);
-
-      /* If this is a top-level window, add the title bar to the region */
-      if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_TOPLEVEL)
-	{
-	  HRGN tmp = CreateRectRgn (0, 0, rect.right - rect.left, -rect.top);
-	  CombineRgn (hrgn, hrgn, tmp, RGN_OR);
-	  DeleteObject (tmp);
-	}
-      
-      SetWindowRgn (GDK_WINDOW_HWND (window), hrgn, TRUE);
+      do_shape_combine_region (window, hrgn, x, y);
 
       private->shaped = TRUE;
     }
 }
 
+void 
+gdk_window_input_shape_combine_mask (GdkWindow *window,
+				     GdkBitmap *mask,
+				     gint       x,
+				     gint       y)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  /* Not yet implemented
+   *
+   * I don't think there is anything in the Win32 API to directly
+   * support this. And anyway, as we don't currently support RGBA
+   * windows, it doesn't really matter.
+   *
+   * When we do support RGBA, input shape functionality could probably
+   * be implemented by saving the input shape region in the per-window
+   * private data, and then simply checking before generating an input
+   * event whether the event's coordinates are inside the region.
+   */
+}
+
 void
 gdk_window_set_override_redirect (GdkWindow *window,
 				  gboolean   override_redirect)
@@ -2820,6 +3030,26 @@ gdk_window_merge_child_shapes (GdkWindow *window)
   gdk_propagate_shapes (GDK_WINDOW_HWND (window), TRUE);
 }
 
+void 
+gdk_window_set_child_input_shapes (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  
+  /* Not yet implemented. See comment in
+   * gdk_window_input_shape_combine_mask().
+   */
+}
+
+void 
+gdk_window_merge_child_input_shapes (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  
+  /* Not yet implemented. See comment in
+   * gdk_window_input_shape_combine_mask().
+   */
+}
+
 gboolean 
 gdk_window_set_static_gravities (GdkWindow *window,
 				 gboolean   use_static)
@@ -3270,18 +3500,65 @@ gdk_window_set_type_hint (GdkWindow        *window,
   update_style_bits (window);
 }
 
+GdkWindowTypeHint
+gdk_window_get_type_hint (GdkWindow *window)
+{
+  g_return_val_if_fail (GDK_IS_WINDOW (window), GDK_WINDOW_TYPE_HINT_NORMAL);
+  
+  if (GDK_WINDOW_DESTROYED (window))
+    return GDK_WINDOW_TYPE_HINT_NORMAL;
+
+  return GDK_WINDOW_IMPL_WIN32 (((GdkWindowObject *) window)->impl)->type_hint;
+}
+
 void
 gdk_window_shape_combine_region (GdkWindow *window,
                                  GdkRegion *shape_region,
                                  gint       offset_x,
                                  gint       offset_y)
 {
+  GdkWindowObject *private = (GdkWindowObject *)window;
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
 
-  /* XXX: even on X implemented conditional ... */  
+  if (!shape_region)
+    {
+      GDK_NOTE (MISC, g_print ("gdk_window_shape_combine_region: %p: none\n",
+			       GDK_WINDOW_HWND (window)));
+      SetWindowRgn (GDK_WINDOW_HWND (window), NULL, TRUE);
+
+      private->shaped = FALSE;
+    }
+  else
+    {
+      HRGN hrgn;
+
+      hrgn = _gdk_win32_gdkregion_to_hrgn (shape_region, 0, 0);
+      
+      GDK_NOTE (MISC, g_print ("gdk_window_shape_combine_region: %p: %p\n",
+			       GDK_WINDOW_HWND (window),
+			       hrgn));
+
+      do_shape_combine_region (window, hrgn, offset_x, offset_y);
+
+      private->shaped = TRUE;
+    }
+}
+
+void 
+gdk_window_input_shape_combine_region (GdkWindow *window,
+				       GdkRegion *shape_region,
+				       gint       offset_x,
+				       gint       offset_y)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  /* Not yet implemented. See comment in
+   * gdk_window_input_shape_combine_mask().
+   */
 }
 
 GdkWindow *
@@ -3295,9 +3572,11 @@ gdk_window_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
 void
 gdk_window_enable_synchronized_configure (GdkWindow *window)
 {
+  g_return_if_fail (GDK_IS_WINDOW (window));
 }
 
 void
 gdk_window_configure_finished (GdkWindow *window)
 {
+  g_return_if_fail (GDK_IS_WINDOW (window));
 }
