@@ -24,10 +24,11 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#include "config.h"
 
 #include "gdkx.h"
 #include "gdkinput.h"
@@ -35,6 +36,7 @@
 #include "gdkinputprivate.h"
 #include "gdkscreen-x11.h"
 #include "gdkdisplay-x11.h"
+#include "gdkalias.h"
 
 static GdkDeviceAxis gdk_input_core_axes[] = {
   { GDK_AXIS_X, 0, 0 },
@@ -69,20 +71,20 @@ gdk_device_get_type (void)
   if (!object_type)
     {
       static const GTypeInfo object_info =
-      {
-        sizeof (GdkDeviceClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) NULL,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (GdkDevicePrivate),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) NULL,
-      };
+	{
+	  sizeof (GdkDeviceClass),
+	  (GBaseInitFunc) NULL,
+	  (GBaseFinalizeFunc) NULL,
+	  (GClassInitFunc) NULL,
+	  NULL,           /* class_finalize */
+	  NULL,           /* class_data */
+	  sizeof (GdkDevicePrivate),
+	  0,              /* n_preallocs */
+	  (GInstanceInitFunc) NULL,
+	};
       
       object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "GdkDevice",
+                                            g_intern_static_string ("GdkDevice"),
                                             &object_info, 0);
     }
   
@@ -105,12 +107,14 @@ gdk_devices_list (void)
 
 /**
  * gdk_display_list_devices:
- * @display : a #GdkDisplay
+ * @display: a #GdkDisplay
  *
  * Returns the list of available input devices attached to @display.
  * The list is statically allocated and should not be freed.
  * 
  * Return value: a list of #GdkDevice
+ *
+ * Since: 2.2
  **/
 GList * 
 gdk_display_list_devices (GdkDisplay *display)
@@ -441,3 +445,6 @@ gdk_device_get_axis (GdkDevice  *device,
   
   return FALSE;
 }
+
+#define __GDK_INPUT_C__
+#include "gdkaliasdef.c"
