@@ -141,10 +141,15 @@ gboolean _gdk_moveresize_handle_event   (XEvent     *event);
 gboolean _gdk_moveresize_configure_done (GdkDisplay *display,
 					 GdkWindow  *window);
 
-void _gdk_keymap_state_changed    (GdkDisplay      *display);
+void _gdk_keymap_state_changed    (GdkDisplay      *display,
+				   XEvent          *event);
 void _gdk_keymap_keys_changed     (GdkDisplay      *display);
 gint _gdk_x11_get_group_for_state (GdkDisplay      *display,
 				   GdkModifierType  state);
+void _gdk_keymap_add_virtual_modifiers (GdkKeymap       *keymap,
+					GdkModifierType *modifiers);
+gboolean _gdk_keymap_key_is_modifier   (GdkKeymap       *keymap,
+					guint            keycode);
 
 GC _gdk_x11_gc_flush (GdkGC *gc);
 
@@ -167,6 +172,7 @@ void _gdk_x11_events_init_screen   (GdkScreen *screen);
 void _gdk_x11_events_uninit_screen (GdkScreen *screen);
 
 void _gdk_events_init           (GdkDisplay *display);
+void _gdk_events_uninit         (GdkDisplay *display);
 void _gdk_windowing_window_init (GdkScreen *screen);
 void _gdk_visual_init           (GdkScreen *screen);
 void _gdk_dnd_init		(GdkDisplay *display);
