@@ -23,9 +23,7 @@
 #include <atk/atk.h>
 #include <gtk/gtkwidget.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 #define GTK_TYPE_ACCESSIBLE                  (gtk_accessible_get_type ())
 #define GTK_ACCESSIBLE(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_ACCESSIBLE, GtkAccessible))
@@ -69,10 +67,7 @@ GType gtk_accessible_get_type (void) G_GNUC_CONST;
 
 void gtk_accessible_connect_widget_destroyed    (GtkAccessible     *accessible);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __GTK_ACCESSIBLE_H__ */
 
