@@ -28,6 +28,7 @@
 #include "gtkcellrenderertext.h"
 #include "gtkcombobox.h"
 #include "gtkcomboboxentry.h"
+#include "gtkprivate.h"
 #include "gtkalias.h"
 
 static void gtk_cell_renderer_combo_class_init (GtkCellRendererComboClass *klass);
@@ -60,7 +61,7 @@ enum {
 
 #define GTK_CELL_RENDERER_COMBO_PATH "gtk-cell-renderer-combo-path"
 
-G_DEFINE_TYPE (GtkCellRendererCombo, gtk_cell_renderer_combo, GTK_TYPE_CELL_RENDERER_TEXT);
+G_DEFINE_TYPE (GtkCellRendererCombo, gtk_cell_renderer_combo, GTK_TYPE_CELL_RENDERER_TEXT)
 
 static void
 gtk_cell_renderer_combo_class_init (GtkCellRendererComboClass *klass)
@@ -88,7 +89,7 @@ gtk_cell_renderer_combo_class_init (GtkCellRendererComboClass *klass)
 							P_("Model"),
 							P_("The model containing the possible values for the combo box"),
 							GTK_TYPE_TREE_MODEL,
-							G_PARAM_READWRITE));
+							GTK_PARAM_READWRITE));
 
   /**
    * GtkCellRendererCombo:text-column:
@@ -105,13 +106,13 @@ gtk_cell_renderer_combo_class_init (GtkCellRendererComboClass *klass)
    */
   g_object_class_install_property (object_class,
                                    PROP_TEXT_COLUMN,
-                                   g_param_spec_int ("text_column",
+                                   g_param_spec_int ("text-column",
                                                      P_("Text Column"),
                                                      P_("A column in the data source model to get the strings from"),
                                                      -1,
                                                      G_MAXINT,
                                                      -1,
-                                                     G_PARAM_READWRITE));
+                                                     GTK_PARAM_READWRITE));
 
   /** 
    * GtkCellRendererCombo:has-entry:
@@ -123,11 +124,11 @@ gtk_cell_renderer_combo_class_init (GtkCellRendererComboClass *klass)
    */
   g_object_class_install_property (object_class,
                                    PROP_HAS_ENTRY,
-                                   g_param_spec_boolean ("has_entry",
+                                   g_param_spec_boolean ("has-entry",
 							 P_("Has Entry"),
 							 P_("If FALSE, don't allow to enter strings other than the chosen ones"),
 							 TRUE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
 
 }
 
@@ -375,9 +376,9 @@ gtk_cell_renderer_combo_start_editing (GtkCellRenderer     *cell,
 				       &(search_data.iter));
     }
 
-  g_object_set (combo, "has_frame", FALSE, NULL);
+  g_object_set (combo, "has-frame", FALSE, NULL);
   g_object_set_data_full (G_OBJECT (combo),
-			  GTK_CELL_RENDERER_COMBO_PATH,
+			  I_(GTK_CELL_RENDERER_COMBO_PATH),
 			  g_strdup (path), g_free);
 
   gtk_widget_show (combo);
