@@ -34,10 +34,7 @@
 #include <gtk/gtkadjustment.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_CONTAINER              (gtk_container_get_type ())
 #define GTK_CONTAINER(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_CONTAINER, GtkContainer))
@@ -59,6 +56,8 @@ struct _GtkContainer
   GtkWidget *focus_child;
   
   guint border_width : 16;
+
+  /*< private >*/
   guint need_resize : 1;
   guint resize_mode : 2;
   guint reallocate_redraws : 1;
@@ -172,15 +171,15 @@ GParamSpec** gtk_container_class_list_child_properties	(GObjectClass	   *cclass,
 void         gtk_container_add_with_properties		(GtkContainer	   *container,
 							 GtkWidget	   *widget,
 							 const gchar	   *first_prop_name,
-							 ...);
+							 ...) G_GNUC_NULL_TERMINATED;
 void         gtk_container_child_set			(GtkContainer	   *container,
 							 GtkWidget	   *child,
 							 const gchar	   *first_prop_name,
-							 ...);	   
+							 ...) G_GNUC_NULL_TERMINATED;
 void         gtk_container_child_get			(GtkContainer	   *container,
 							 GtkWidget	   *child,
 							 const gchar	   *first_prop_name,
-							 ...);	   
+							 ...) G_GNUC_NULL_TERMINATED;
 void         gtk_container_child_set_valist		(GtkContainer	   *container,
 							 GtkWidget	   *child,
 							 const gchar	   *first_property_name,
@@ -221,9 +220,6 @@ GList *_gtk_container_focus_sort             (GtkContainer     *container,
 #define	gtk_container_border_width		gtk_container_set_border_width
 #endif /* GTK_DISABLE_DEPRECATED */
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __GTK_CONTAINER_H__ */
