@@ -128,7 +128,7 @@ GtkWidget* gtk_dialog_new_with_buttons (const gchar     *title,
                                         GtkWindow       *parent,
                                         GtkDialogFlags   flags,
                                         const gchar     *first_button_text,
-                                        ...) G_GNUC_NULL_TERMINATED;
+                                        ...);
 
 void       gtk_dialog_add_action_widget (GtkDialog   *dialog,
                                          GtkWidget   *child,
@@ -171,8 +171,6 @@ gint gtk_dialog_run                (GtkDialog *dialog);
 /* For private use only */
 void _gtk_dialog_set_ignore_separator (GtkDialog *dialog,
 				       gboolean   ignore_separator);
-gint _gtk_dialog_get_response_for_widget (GtkDialog *dialog,
-					  GtkWidget *widget);
 
 G_END_DECLS
 
