@@ -43,9 +43,8 @@
 #include "gtkstock.h"
 #include "gtkwindow.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
-
 #include "gtkquartz.h"
+#include "gtkalias.h"
 #include "gdk/quartz/gdkquartz.h"
 
 typedef struct _GtkDragSourceSite GtkDragSourceSite;
@@ -105,6 +104,7 @@ struct _GtkDragDestSite
   GtkTargetList     *target_list;
   GdkDragAction      actions;
   guint              have_drag : 1;
+  guint              track_motion : 1;
 };
 
 struct _GtkDragDestInfo 
@@ -281,6 +281,7 @@ gtk_drag_get_source_info (GdkDragContext *context,
 			  gboolean        create)
 {
   GtkDragSourceInfo *info;
+
   if (!dest_info_quark)
     dest_info_quark = g_quark_from_static_string ("gtk-source-info");
   
@@ -417,7 +418,7 @@ register_types (GtkWidget *widget, GtkDragDestSite *site)
       if (!nswindow)
 	return;
 
-      pool= [[NSAutoreleasePool alloc] init];
+      pool = [[NSAutoreleasePool alloc] init];
       types = _gtk_quartz_target_list_to_pasteboard_types (site->target_list);
 
       [nswindow registerForDraggedTypes:types];
@@ -467,15 +468,8 @@ gtk_drag_dest_set (GtkWidget            *widget,
   g_return_if_fail (GTK_IS_WIDGET (widget));
 
   old_site = g_object_get_data (G_OBJECT (widget), "gtk-drag-dest");
-  if (old_site)
-    {
-      g_signal_handlers_disconnect_by_func (widget,
-					    gtk_drag_dest_realized,
-					    old_site);
-      g_signal_handlers_disconnect_by_func (widget,
-					    gtk_drag_dest_hierarchy_changed,
-					    old_site);
-    }
+
+  gtk_drag_dest_unset (widget);
 
   site = g_new (GtkDragDestSite, 1);
   site->flags = flags;
@@ -486,6 +480,11 @@ gtk_drag_dest_set (GtkWidget            *widget,
     site->target_list = NULL;
   site->actions = actions;
 
+  if (old_site)
+    site->track_motion = old_site->track_motion;
+  else
+    site->track_motion = FALSE;
+
   if (GTK_WIDGET_REALIZED (widget))
     gtk_drag_dest_realized (widget, site);
 
@@ -510,8 +509,21 @@ gtk_drag_dest_set_proxy (GtkWidget      *widget,
 void 
 gtk_drag_dest_unset (GtkWidget *widget)
 {
+  GtkDragDestSite *old_site;
+
   g_return_if_fail (GTK_IS_WIDGET (widget));
 
+  old_site = g_object_get_data (G_OBJECT (widget), "gtk-drag-dest");
+  if (old_site)
+    {
+      g_signal_handlers_disconnect_by_func (widget,
+                                            gtk_drag_dest_realized,
+                                            old_site);
+      g_signal_handlers_disconnect_by_func (widget,
+                                            gtk_drag_dest_hierarchy_changed,
+                                            old_site);
+    }
+
   g_object_set_data (G_OBJECT (widget), I_("gtk-drag-dest"), NULL);
 }
 
@@ -756,9 +768,9 @@ gtk_drag_dest_leave (GtkWidget      *widget,
   if ((site->flags & GTK_DEST_DEFAULT_HIGHLIGHT) && site->have_drag)
     gtk_drag_unhighlight (widget);
   
-  if (!(site->flags & GTK_DEST_DEFAULT_MOTION) || site->have_drag)
-    g_signal_emit_by_name (widget, "drag_leave",
-			   context, time);
+  if (!(site->flags & GTK_DEST_DEFAULT_MOTION) || site->have_drag ||
+      site->track_motion)
+    g_signal_emit_by_name (widget, "drag_leave", context, time);
   
   site->have_drag = FALSE;
 }
@@ -777,7 +789,7 @@ gtk_drag_dest_motion (GtkWidget	     *widget,
   site = g_object_get_data (G_OBJECT (widget), "gtk-drag-dest");
   g_return_val_if_fail (site != NULL, FALSE);
 
-  if (site->flags & GTK_DEST_DEFAULT_MOTION)
+  if (site->track_motion || site->flags & GTK_DEST_DEFAULT_MOTION)
     {
       if (context->suggested_action & site->actions)
 	action = context->suggested_action;
@@ -796,7 +808,8 @@ gtk_drag_dest_motion (GtkWidget	     *widget,
       else
 	{
 	  gdk_drag_status (context, 0, time);
-	  return TRUE;
+	  if (!site->track_motion)
+	    return TRUE;
 	}
     }
 
@@ -849,17 +862,28 @@ void
 gtk_drag_dest_set_track_motion (GtkWidget *widget,
 				gboolean   track_motion)
 {
+  GtkDragDestSite *site;
+
   g_return_if_fail (GTK_IS_WIDGET (widget));
 
-  /* FIXME: Implement */
+  site = g_object_get_data (G_OBJECT (widget), "gtk-drag-dest");
+  
+  g_return_if_fail (site != NULL);
+
+  site->track_motion = track_motion != FALSE;
 }
 
 gboolean
 gtk_drag_dest_get_track_motion (GtkWidget *widget)
 {
+  GtkDragDestSite *site;
+
   g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
-  /* FIXME: Implement */
+  site = g_object_get_data (G_OBJECT (widget), "gtk-drag-dest");
+
+  if (site)
+    return site->track_motion;
 
   return FALSE;
 }
@@ -1040,11 +1064,28 @@ gtk_drag_begin_internal (GtkWidget         *widget,
 	switch (site->icon_type)
 	  {
 	  case GTK_IMAGE_PIXMAP:
-	    gtk_drag_set_icon_pixmap (context,
-				      site->colormap,
-				      site->icon_data.pixmap.pixmap,
-				      site->icon_mask,
-				      -2, -2);
+	    /* This is not supported, so just set a small transparent pixbuf
+	     * since we need to have something.
+	     */
+	    if (0)
+	      gtk_drag_set_icon_pixmap (context,
+					site->colormap,
+					site->icon_data.pixmap.pixmap,
+					site->icon_mask,
+					-2, -2);
+	    else
+	      {
+		GdkPixbuf *pixbuf;
+
+		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, 1, 1);
+		gdk_pixbuf_fill (pixbuf, 0xffffff);
+	    
+		gtk_drag_set_icon_pixbuf (context,
+					  pixbuf,
+					  0, 0);
+
+		g_object_unref (pixbuf);
+	      }
 	    break;
 	  case GTK_IMAGE_PIXBUF:
 	    gtk_drag_set_icon_pixbuf (context,
