@@ -354,7 +354,7 @@ gtk_file_chooser_button_class_init (GtkFileChooserButtonClass * class)
 				   g_param_spec_object ("dialog",
 							P_("Dialog"),
 							P_("The file chooser dialog to use."),
-							GTK_TYPE_FILE_CHOOSER_DIALOG,
+							GTK_TYPE_FILE_CHOOSER,
 							(GTK_PARAM_WRITABLE |
 							 G_PARAM_CONSTRUCT_ONLY)));
 
@@ -797,7 +797,6 @@ gtk_file_chooser_button_set_property (GObject      *object,
       priv->has_title = TRUE;
       /* Intentionally fall through instead of breaking here, to actually set the property. */
     case GTK_FILE_CHOOSER_PROP_FILTER:
-    case GTK_FILE_CHOOSER_PROP_LOCAL_ONLY:
     case GTK_FILE_CHOOSER_PROP_PREVIEW_WIDGET:
     case GTK_FILE_CHOOSER_PROP_PREVIEW_WIDGET_ACTIVE:
     case GTK_FILE_CHOOSER_PROP_USE_PREVIEW_LABEL:
@@ -807,6 +806,12 @@ gtk_file_chooser_button_set_property (GObject      *object,
       g_object_set_property (G_OBJECT (priv->dialog), pspec->name, value);
       break;
 
+    case GTK_FILE_CHOOSER_PROP_LOCAL_ONLY:
+      g_object_set_property (G_OBJECT (priv->dialog), pspec->name, value);
+      fs_volumes_changed_cb (priv->fs, button);
+      fs_bookmarks_changed_cb (priv->fs, button);
+      break;
+
     case GTK_FILE_CHOOSER_PROP_FILE_SYSTEM_BACKEND:
       /* Construct-only */
       priv->backend = g_value_dup_string (value);
@@ -1212,17 +1217,20 @@ change_icon_theme_get_info_cb (GtkFileSystemHandle *handle,
       width = MAX (width, gdk_pixbuf_get_width (pixbuf));
 
       path = gtk_tree_row_reference_get_path (data->row_ref);
-      gtk_tree_model_get_iter (data->button->priv->model, &iter, path);
-      gtk_tree_path_free (path);
+      if (path) 
+        {
+          gtk_tree_model_get_iter (data->button->priv->model, &iter, path);
+          gtk_tree_path_free (path);
 
-      gtk_list_store_set (GTK_LIST_STORE (data->button->priv->model), &iter,
-			  ICON_COLUMN, pixbuf,
-			  -1);
+          gtk_list_store_set (GTK_LIST_STORE (data->button->priv->model), &iter,
+	  		      ICON_COLUMN, pixbuf,
+			      -1);
+
+          g_object_set (data->button->priv->icon_cell,
+		        "width", width,
+		        NULL);
+        }
       g_object_unref (pixbuf);
-
-      g_object_set (data->button->priv->icon_cell,
-		    "width", width,
-		    NULL);
     }
 
 out:
@@ -1285,23 +1293,34 @@ change_icon_theme (GtkFileChooserButton *button)
 	case ROW_TYPE_CURRENT_FOLDER:
 	  if (data)
 	    {
-	      GtkTreePath *path;
-	      GtkFileSystemHandle *handle;
-	      struct ChangeIconThemeData *info;
-
-	      info = g_new0 (struct ChangeIconThemeData, 1);
-	      info->button = g_object_ref (button);
-	      path = gtk_tree_model_get_path (priv->model, &iter);
-	      info->row_ref = gtk_tree_row_reference_new (priv->model, path);
-	      gtk_tree_path_free (path);
-
-	      handle =
-	        gtk_file_system_get_info (priv->fs, data, GTK_FILE_INFO_ICON,
-					  change_icon_theme_get_info_cb,
-					  info);
-	      button->priv->change_icon_theme_handles =
-		g_slist_append (button->priv->change_icon_theme_handles, handle);
-	      pixbuf = NULL;
+	      if (gtk_file_system_path_is_local (priv->fs, (GtkFilePath *)data))
+		{
+		  GtkTreePath *path;
+		  GtkFileSystemHandle *handle;
+		  struct ChangeIconThemeData *info;		  
+		  
+		  info = g_new0 (struct ChangeIconThemeData, 1);
+		  info->button = g_object_ref (button);
+		  path = gtk_tree_model_get_path (priv->model, &iter);
+		  info->row_ref = gtk_tree_row_reference_new (priv->model, path);
+		  gtk_tree_path_free (path);
+		  
+		  handle =
+		    gtk_file_system_get_info (priv->fs, data, GTK_FILE_INFO_ICON,
+					      change_icon_theme_get_info_cb,
+					      info);
+		  button->priv->change_icon_theme_handles =
+		    g_slist_append (button->priv->change_icon_theme_handles, handle);
+		  pixbuf = NULL;
+		}
+	      else
+		/* Don't call get_info for remote paths to avoid latency and
+		 * auth dialogs.
+		 * If we switch to a better bookmarks file format (XBEL), we
+		 * should use mime info to get a better icon.
+		 */
+		pixbuf = gtk_icon_theme_load_icon (theme, "gnome-fs-regular",
+						   priv->icon_size, 0, NULL);
 	    }
 	  else
 	    pixbuf = gtk_icon_theme_load_icon (theme, FALLBACK_ICON_NAME,
@@ -1381,6 +1400,7 @@ get_icon_theme (GtkWidget *widget)
 struct SetDisplayNameData
 {
   GtkFileChooserButton *button;
+  char *label;
   GtkTreeRowReference *row_ref;
 };
 
@@ -1397,6 +1417,10 @@ set_info_get_info_cb (GtkFileSystemHandle *handle,
   GtkFileSystemHandle *model_handle;
   struct SetDisplayNameData *data = callback_data;
 
+  if (!data->button->priv->model)
+    /* button got destroyed */
+    goto out;
+
   path = gtk_tree_row_reference_get_path (data->row_ref);
   if (!path)
     /* Handle doesn't exist anymore in the model */
@@ -1423,9 +1447,12 @@ set_info_get_info_cb (GtkFileSystemHandle *handle,
   pixbuf = gtk_file_info_render_icon (info, GTK_WIDGET (data->button),
 				      data->button->priv->icon_size, NULL);
 
+  if (!data->label)
+    data->label = g_strdup (gtk_file_info_get_display_name (info));
+
   gtk_list_store_set (GTK_LIST_STORE (data->button->priv->model), &iter,
 		      ICON_COLUMN, pixbuf,
-		      DISPLAY_NAME_COLUMN, gtk_file_info_get_display_name (info),
+		      DISPLAY_NAME_COLUMN, data->label,
 		      IS_FOLDER_COLUMN, gtk_file_info_get_is_folder (info),
 		      -1);
 
@@ -1434,6 +1461,7 @@ set_info_get_info_cb (GtkFileSystemHandle *handle,
 
 out:
   g_object_unref (data->button);
+  g_free (data->label);
   gtk_tree_row_reference_free (data->row_ref);
   g_free (data);
 
@@ -1451,6 +1479,7 @@ set_info_for_path_at_iter (GtkFileChooserButton *button,
 
   data = g_new0 (struct SetDisplayNameData, 1);
   data->button = g_object_ref (button);
+  data->label = gtk_file_system_get_bookmark_label (button->priv->fs, path);
 
   tree_path = gtk_tree_model_get_path (button->priv->model, iter);
   data->row_ref = gtk_tree_row_reference_new (button->priv->model, tree_path);
@@ -1565,6 +1594,10 @@ model_add_special_get_info_cb (GtkFileSystemHandle *handle,
   GtkFileSystemHandle *model_handle;
   struct ChangeIconThemeData *data = user_data;
 
+  if (!data->button->priv->model)
+    /* button got destroyed */
+    goto out;
+
   path = gtk_tree_row_reference_get_path (data->row_ref);
   if (!path)
     /* Handle doesn't exist anymore in the model */
@@ -1684,7 +1717,7 @@ model_add_special (GtkFileChooserButton *button)
       gtk_tree_path_free (tree_path);
 
       handle = gtk_file_system_get_info (button->priv->fs, path,
-					 GTK_FILE_INFO_ICON,
+					 GTK_FILE_INFO_DISPLAY_NAME | GTK_FILE_INFO_ICON,
 					 model_add_special_get_info_cb, info);
 
       gtk_list_store_set (store, &iter,
@@ -1693,6 +1726,7 @@ model_add_special (GtkFileChooserButton *button)
 			  DISPLAY_NAME_COLUMN, _(DESKTOP_DISPLAY_NAME),
 			  DATA_COLUMN, path,
 			  IS_FOLDER_COLUMN, TRUE,
+			  HANDLE_COLUMN, handle,
 			  -1);
 
       button->priv->n_special++;
@@ -1705,33 +1739,61 @@ model_add_volumes (GtkFileChooserButton *button,
 {
   GtkListStore *store;
   gint pos;
-
+  gboolean local_only;
+  GtkFileSystem *file_system;
+  GSList *l;
+  
   if (!volumes)
     return;
 
   store = GTK_LIST_STORE (button->priv->model);
   pos = model_get_type_position (button, ROW_TYPE_VOLUME);
+  local_only = gtk_file_chooser_get_local_only (GTK_FILE_CHOOSER (button->priv->dialog));
+  file_system = button->priv->fs;
 
-  do
+  for (l = volumes; l; l = l->next)
     {
+      GtkFileSystemVolume *volume;
       GtkTreeIter iter;
       GdkPixbuf *pixbuf;
       gchar *display_name;
 
-      pixbuf = gtk_file_system_volume_render_icon (button->priv->fs,
-						   volumes->data,
+      volume = l->data;
+
+      if (local_only)
+	{
+	  if (gtk_file_system_volume_get_is_mounted (file_system, volume))
+	    {
+	      GtkFilePath *base_path;
+
+	      base_path = gtk_file_system_volume_get_base_path (file_system, volume);
+	      if (base_path != NULL)
+		{
+		  gboolean is_local = gtk_file_system_path_is_local (file_system, base_path);
+		  gtk_file_path_free (base_path);
+
+		  if (!is_local)
+		    {
+		      gtk_file_system_volume_free (file_system, volume);
+		      continue;
+		    }
+		}
+	    }
+	}
+
+      pixbuf = gtk_file_system_volume_render_icon (file_system,
+						   volume,
 						   GTK_WIDGET (button),
 						   button->priv->icon_size,
 						   NULL);
-      display_name = gtk_file_system_volume_get_display_name (button->priv->fs,
-							      volumes->data);
+      display_name = gtk_file_system_volume_get_display_name (file_system, volume);
 
       gtk_list_store_insert (store, &iter, pos);
       gtk_list_store_set (store, &iter,
 			  ICON_COLUMN, pixbuf,
 			  DISPLAY_NAME_COLUMN, display_name,
 			  TYPE_COLUMN, ROW_TYPE_VOLUME,
-			  DATA_COLUMN, volumes->data,
+			  DATA_COLUMN, volume,
 			  IS_FOLDER_COLUMN, TRUE,
 			  -1);
 
@@ -1741,11 +1803,11 @@ model_add_volumes (GtkFileChooserButton *button,
 
       button->priv->n_volumes++;
       pos++;
-      volumes = volumes->next;
     }
-  while (volumes);
 }
 
+extern gchar * _gtk_file_chooser_label_for_uri (const gchar *uri);
+
 static void
 model_add_bookmarks (GtkFileChooserButton *button,
 		     GSList               *bookmarks)
@@ -1753,15 +1815,84 @@ model_add_bookmarks (GtkFileChooserButton *button,
   GtkListStore *store;
   GtkTreeIter iter;
   gint pos;
+  gboolean local_only;
+  GSList *l;
 
   if (!bookmarks)
     return;
 
   store = GTK_LIST_STORE (button->priv->model);
-  pos = model_get_type_position (button, ROW_TYPE_BOOKMARK_SEPARATOR);
+  pos = model_get_type_position (button, ROW_TYPE_BOOKMARK);
+  local_only = gtk_file_chooser_get_local_only (GTK_FILE_CHOOSER (button->priv->dialog));
 
-  if (!button->priv->has_bookmark_separator)
+  for (l = bookmarks; l; l = l->next)
     {
+      GtkFilePath *path;
+
+      path = l->data;
+
+      if (gtk_file_system_path_is_local (button->priv->fs, path))
+	{
+	  gtk_list_store_insert (store, &iter, pos);
+	  gtk_list_store_set (store, &iter,
+			      ICON_COLUMN, NULL,
+			      DISPLAY_NAME_COLUMN, _(FALLBACK_DISPLAY_NAME),
+			      TYPE_COLUMN, ROW_TYPE_BOOKMARK,
+			      DATA_COLUMN, gtk_file_path_copy (path),
+			      IS_FOLDER_COLUMN, FALSE,
+			      -1);
+	  set_info_for_path_at_iter (button, path, &iter);
+	}
+      else
+	{
+	  gchar *label;
+	  GtkIconTheme *icon_theme;
+	  GdkPixbuf *pixbuf;
+
+	  if (local_only)
+	    continue;
+
+	  /* Don't call get_info for remote paths to avoid latency and
+	   * auth dialogs.
+	   * If we switch to a better bookmarks file format (XBEL), we
+	   * should use mime info to get a better icon.
+	   */
+	  label = gtk_file_system_get_bookmark_label (button->priv->fs, path);
+	  if (!label)
+	    {
+	      gchar *uri;
+
+	      uri = gtk_file_system_path_to_uri (button->priv->fs, path);
+	      label = _gtk_file_chooser_label_for_uri (uri);
+	      g_free (uri);
+	    }
+
+	  icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (button)));
+	  pixbuf = gtk_icon_theme_load_icon (icon_theme, "gnome-fs-directory", 
+					     button->priv->icon_size, 0, NULL);
+
+	  gtk_list_store_insert (store, &iter, pos);
+	  gtk_list_store_set (store, &iter,
+			      ICON_COLUMN, pixbuf,
+			      DISPLAY_NAME_COLUMN, label,
+			      TYPE_COLUMN, ROW_TYPE_BOOKMARK,
+			      DATA_COLUMN, gtk_file_path_copy (path),
+			      IS_FOLDER_COLUMN, TRUE,
+			      -1);
+
+	  g_free (label);
+	  g_object_unref (pixbuf);
+	}
+
+      button->priv->n_bookmarks++;
+      pos++;
+    }
+
+  if (button->priv->n_bookmarks > 0 && 
+      !button->priv->has_bookmark_separator)
+    {
+      pos = model_get_type_position (button, ROW_TYPE_BOOKMARK_SEPARATOR);
+
       gtk_list_store_insert (store, &iter, pos);
       gtk_list_store_set (store, &iter,
 			  ICON_COLUMN, NULL,
@@ -1772,25 +1903,6 @@ model_add_bookmarks (GtkFileChooserButton *button,
 			  -1);
       button->priv->has_bookmark_separator = TRUE;
     }
-
-  do
-    {
-      pos++;
-
-      gtk_list_store_insert (store, &iter, pos);
-      gtk_list_store_set (store, &iter,
-			  ICON_COLUMN, NULL,
-			  DISPLAY_NAME_COLUMN, _(FALLBACK_DISPLAY_NAME),
-			  TYPE_COLUMN, ROW_TYPE_BOOKMARK,
-			  DATA_COLUMN, gtk_file_path_copy (bookmarks->data),
-			  IS_FOLDER_COLUMN, FALSE,
-			  -1);
-      set_info_for_path_at_iter (button, bookmarks->data, &iter);
-
-      button->priv->n_bookmarks++;
-      bookmarks = bookmarks->next;
-    }
-  while (bookmarks);
 }
 
 static void
@@ -1832,14 +1944,53 @@ model_update_current_folder (GtkFileChooserButton *button,
       model_free_row_data (button, &iter);
     }
 
-  gtk_list_store_set (store, &iter,
-		      ICON_COLUMN, NULL,
-		      DISPLAY_NAME_COLUMN, _(FALLBACK_DISPLAY_NAME),
-		      TYPE_COLUMN, ROW_TYPE_CURRENT_FOLDER,
-		      DATA_COLUMN, gtk_file_path_copy (path),
-		      IS_FOLDER_COLUMN, FALSE,
-		      -1);
-  set_info_for_path_at_iter (button, path, &iter);
+  if (gtk_file_system_path_is_local (button->priv->fs, path))
+    {
+      gtk_list_store_set (store, &iter,
+			  ICON_COLUMN, NULL,
+			  DISPLAY_NAME_COLUMN, _(FALLBACK_DISPLAY_NAME),
+			  TYPE_COLUMN, ROW_TYPE_CURRENT_FOLDER,
+			  DATA_COLUMN, gtk_file_path_copy (path),
+			  IS_FOLDER_COLUMN, FALSE,
+			  -1);
+      set_info_for_path_at_iter (button, path, &iter);
+    }
+  else
+    {
+      gchar *label;
+      GtkIconTheme *icon_theme;
+      GdkPixbuf *pixbuf;
+
+      /* Don't call get_info for remote paths to avoid latency and
+       * auth dialogs.
+       * If we switch to a better bookmarks file format (XBEL), we
+       * should use mime info to get a better icon.
+       */
+      label = gtk_file_system_get_bookmark_label (button->priv->fs, path);
+      if (!label)
+	{
+	  gchar *uri;
+	  
+	  uri = gtk_file_system_path_to_uri (button->priv->fs, path);
+	  label = _gtk_file_chooser_label_for_uri (uri);
+	  g_free (uri);
+	}
+      
+      icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (button)));
+      pixbuf = gtk_icon_theme_load_icon (icon_theme, "gnome-fs-directory", 
+					 button->priv->icon_size, 0, NULL);
+      
+      gtk_list_store_set (store, &iter,
+			  ICON_COLUMN, pixbuf,
+			  DISPLAY_NAME_COLUMN, label,
+			  TYPE_COLUMN, ROW_TYPE_CURRENT_FOLDER,
+			  DATA_COLUMN, gtk_file_path_copy (path),
+			  IS_FOLDER_COLUMN, TRUE,
+			  -1);
+      
+      g_free (label);
+      g_object_unref (pixbuf);
+    }
 }
 
 static inline void
@@ -1951,17 +2102,27 @@ filter_model_visible_func (GtkTreeModel *model,
       break;
     case ROW_TYPE_VOLUME:
       {
-	GtkFilePath *base_path;
-
-	base_path = gtk_file_system_volume_get_base_path (priv->fs, data);
-	if (base_path)
+	retval = TRUE;
+	if (local_only)
 	  {
-	    retval = (!local_only ||
-		      gtk_file_system_path_is_local (priv->fs, base_path));
-	    gtk_file_path_free (base_path);
+	    if (gtk_file_system_volume_get_is_mounted (priv->fs, data))
+	      {
+		GtkFilePath *base_path;
+		
+		base_path = gtk_file_system_volume_get_base_path (priv->fs, data);
+		if (base_path)
+		  {
+		    gboolean is_local = gtk_file_system_path_is_local (priv->fs, base_path);
+		    
+		    gtk_file_path_free (base_path);
+
+		    if (!is_local)
+		      retval = FALSE;
+		  }
+		else
+		  retval = FALSE;
+	      }
 	  }
-	else
-	  retval = FALSE;
       }
       break;
     default:
@@ -2049,10 +2210,13 @@ update_combo_box (GtkFileChooserButton *button)
 	    GtkFilePath *base_path;
 
 	    base_path = gtk_file_system_volume_get_base_path (priv->fs, data);
-	    row_found = (paths &&
-			 paths->data &&
-			 gtk_file_path_compare (base_path, paths->data) == 0);
-	    gtk_file_path_free (base_path);
+            if (base_path)
+              {
+	        row_found = (paths &&
+		  	     paths->data &&
+			     gtk_file_path_compare (base_path, paths->data) == 0);
+	        gtk_file_path_free (base_path);
+              }
 	  }
 	  break;
 	default:
@@ -2177,13 +2341,34 @@ update_label_and_image (GtkFileChooserButton *button)
 	}
 
       if (priv->update_button_handle)
-	gtk_file_system_cancel_operation (priv->update_button_handle);
+	{
+	  gtk_file_system_cancel_operation (priv->update_button_handle);
+	  priv->update_button_handle = NULL;
+	}
+	  
+      if (gtk_file_system_path_is_local (priv->fs, path))
+	{
+	  priv->update_button_handle =
+	    gtk_file_system_get_info (priv->fs, path,
+				      GTK_FILE_INFO_DISPLAY_NAME | GTK_FILE_INFO_ICON,
+				      update_label_get_info_cb,
+				      g_object_ref (button));
+	}
+      else
+	{
+	  GdkPixbuf *pixbuf;
 
-      priv->update_button_handle =
-        gtk_file_system_get_info (priv->fs, path,
-				  GTK_FILE_INFO_DISPLAY_NAME | GTK_FILE_INFO_ICON,
-				  update_label_get_info_cb,
-				  g_object_ref (button));
+	  label_text = gtk_file_system_get_bookmark_label (button->priv->fs, path);
+	  
+	  pixbuf = gtk_icon_theme_load_icon (get_icon_theme (GTK_WIDGET (priv->image)), 
+					     "gnome-fs-regular",
+					     priv->icon_size, 0, NULL);
+	  
+	  gtk_image_set_from_pixbuf (GTK_IMAGE (priv->image), pixbuf);
+
+	  if (pixbuf)
+	    g_object_unref (pixbuf);
+	}
     }
 out:
   gtk_file_paths_free (paths);
@@ -2236,19 +2421,11 @@ fs_bookmarks_changed_cb (GtkFileSystem *fs,
   GSList *bookmarks;
 
   bookmarks = gtk_file_system_list_bookmarks (fs);
-  if (!bookmarks)
-    {
-      model_remove_rows (user_data,
-			 model_get_type_position (user_data,
-						  ROW_TYPE_BOOKMARK_SEPARATOR),
-			 (priv->n_bookmarks + priv->has_bookmark_separator));
-      priv->has_bookmark_separator = FALSE;
-    }
-  else
-    model_remove_rows (user_data,
-		       model_get_type_position (user_data, ROW_TYPE_BOOKMARK),
-		       priv->n_bookmarks);
-
+  model_remove_rows (user_data,
+		     model_get_type_position (user_data,
+					      ROW_TYPE_BOOKMARK_SEPARATOR),
+		     (priv->n_bookmarks + priv->has_bookmark_separator));
+  priv->has_bookmark_separator = FALSE;
   priv->n_bookmarks = 0;
   model_add_bookmarks (user_data, bookmarks);
   gtk_file_paths_free (bookmarks);
@@ -2581,11 +2758,12 @@ gtk_file_chooser_button_new_with_backend (const gchar          *title,
 
 /**
  * gtk_file_chooser_button_new_with_dialog:
- * @dialog: the #GtkFileChooserDialog widget to use.
+ * @dialog: the widget to use as dialog
  * 
  * Creates a #GtkFileChooserButton widget which uses @dialog as it's
- * file-picking window. Note that @dialog must be a #GtkFileChooserDialog (or
- * subclass) and must not have %GTK_DIALOG_DESTROY_WITH_PARENT set.
+ * file-picking window. Note that @dialog must be a #GtkDialog (or
+ * subclass) which implements the #GtkFileChooser interface and must 
+ * not have %GTK_DIALOG_DESTROY_WITH_PARENT set.
  * 
  * Returns: a new button widget.
  * 
@@ -2594,7 +2772,7 @@ gtk_file_chooser_button_new_with_backend (const gchar          *title,
 GtkWidget *
 gtk_file_chooser_button_new_with_dialog (GtkWidget *dialog)
 {
-  g_return_val_if_fail (GTK_IS_FILE_CHOOSER_DIALOG (dialog), NULL);
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (dialog) && GTK_IS_DIALOG (dialog), NULL);
 
   return g_object_new (GTK_TYPE_FILE_CHOOSER_BUTTON,
 		       "dialog", dialog,
