@@ -43,8 +43,6 @@ struct _GtkMenuToolButtonPrivate
   GtkMenu   *menu;
 };
 
-static void gtk_menu_tool_button_init       (GtkMenuToolButton      *button);
-static void gtk_menu_tool_button_class_init (GtkMenuToolButtonClass *klass);
 static void gtk_menu_tool_button_destroy    (GtkObject              *object);
 
 static int  menu_deactivate_cb              (GtkMenuShell           *menu_shell,
@@ -65,35 +63,7 @@ enum
 
 static gint signals[LAST_SIGNAL];
 
-static GObjectClass *parent_class = NULL;
-
-GType
-gtk_menu_tool_button_get_type (void)
-{
-  static GType type = 0;
-
-  if (type == 0)
-    {
-      static const GTypeInfo info =
-	{
-	  sizeof (GtkMenuToolButtonClass),
-	  (GBaseInitFunc) 0,
-	  (GBaseFinalizeFunc) 0,
-	  (GClassInitFunc) gtk_menu_tool_button_class_init,
-	  (GClassFinalizeFunc) 0,
-	  NULL,
-	  sizeof (GtkMenuToolButton),
-	  0, /* n_preallocs */
-	  (GInstanceInitFunc) gtk_menu_tool_button_init
-	};
-
-      type = g_type_register_static (GTK_TYPE_TOOL_BUTTON,
-                                     "GtkMenuToolButton",
-                                     &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (GtkMenuToolButton, gtk_menu_tool_button, GTK_TYPE_TOOL_BUTTON)
 
 static gboolean
 gtk_menu_tool_button_set_tooltip (GtkToolItem *tool_item,
@@ -175,7 +145,7 @@ gtk_menu_tool_button_toolbar_reconfigured (GtkToolItem *toolitem)
   gtk_menu_tool_button_construct_contents (GTK_MENU_TOOL_BUTTON (toolitem));
 
   /* chain up */
-  GTK_TOOL_ITEM_CLASS (parent_class)->toolbar_reconfigured (toolitem);
+  GTK_TOOL_ITEM_CLASS (gtk_menu_tool_button_parent_class)->toolbar_reconfigured (toolitem);
 }
 
 static void
@@ -238,15 +208,11 @@ gtk_menu_tool_button_class_init (GtkMenuToolButtonClass *klass)
   GtkObjectClass *gtk_object_class;
   GtkWidgetClass *widget_class;
   GtkToolItemClass *toolitem_class;
-  GtkToolButtonClass *toolbutton_class;
-
-  parent_class = g_type_class_peek_parent (klass);
 
   object_class = (GObjectClass *)klass;
   gtk_object_class = (GtkObjectClass *)klass;
   widget_class = (GtkWidgetClass *)klass;
   toolitem_class = (GtkToolItemClass *)klass;
-  toolbutton_class = (GtkToolButtonClass *)klass;
 
   object_class->set_property = gtk_menu_tool_button_set_property;
   object_class->get_property = gtk_menu_tool_button_get_property;
@@ -256,7 +222,7 @@ gtk_menu_tool_button_class_init (GtkMenuToolButtonClass *klass)
   toolitem_class->toolbar_reconfigured = gtk_menu_tool_button_toolbar_reconfigured;
 
   signals[SHOW_MENU] =
-    g_signal_new ("show-menu",
+    g_signal_new (I_("show-menu"),
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GtkMenuToolButtonClass, show_menu),
@@ -452,8 +418,7 @@ gtk_menu_tool_button_destroy (GtkObject *object)
       g_signal_handlers_disconnect_by_func (button->priv->menu, 
 					    menu_deactivate_cb, 
 					    button);
-      g_object_unref (button->priv->menu);
-      button->priv->menu = NULL;
+      gtk_menu_detach (button->priv->menu);
 
       g_signal_handlers_disconnect_by_func (button->priv->arrow_button,
 					    arrow_button_toggled_cb, 
@@ -463,8 +428,8 @@ gtk_menu_tool_button_destroy (GtkObject *object)
 					    button);
     }
   
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  if (GTK_OBJECT_CLASS (gtk_menu_tool_button_parent_class)->destroy)
+    (*GTK_OBJECT_CLASS (gtk_menu_tool_button_parent_class)->destroy) (object);
 }
 
 /**
@@ -537,6 +502,17 @@ menu_deactivate_cb (GtkMenuShell      *menu_shell,
   return TRUE;
 }
 
+static void
+menu_detacher (GtkWidget *widget,
+               GtkMenu   *menu)
+{
+  GtkMenuToolButtonPrivate *priv = GTK_MENU_TOOL_BUTTON (widget)->priv;
+
+  g_return_if_fail (priv->menu == menu);
+
+  priv->menu = NULL;
+}
+
 /**
  * gtk_menu_tool_button_set_menu:
  * @button: a #GtkMenuToolButton
@@ -568,19 +544,19 @@ gtk_menu_tool_button_set_menu (GtkMenuToolButton *button,
           g_signal_handlers_disconnect_by_func (priv->menu, 
 						menu_deactivate_cb, 
 						button);
-	  g_object_unref (priv->menu);
+	  gtk_menu_detach (priv->menu);
 	}
 
       priv->menu = GTK_MENU (menu);
 
       if (priv->menu)
         {
-          g_object_ref (priv->menu);
-          gtk_object_sink (GTK_OBJECT (priv->menu));
+          gtk_menu_attach_to_widget (priv->menu, GTK_WIDGET (button),
+                                     menu_detacher);
 
           gtk_widget_set_sensitive (priv->arrow_button, TRUE);
 
-          g_signal_connect (button->priv->menu, "deactivate",
+          g_signal_connect (priv->menu, "deactivate",
                             G_CALLBACK (menu_deactivate_cb), button);
         }
       else
