@@ -78,6 +78,7 @@ struct GtkPageSetupUnixDialogPrivate
   GtkTooltips *tooltips;
 
   GtkWidget *portrait_radio;
+  GtkWidget *reverse_portrait_radio;
   GtkWidget *landscape_radio;
   GtkWidget *reverse_landscape_radio;
 
@@ -126,7 +127,7 @@ static void printer_status_cb                    (GtkPrintBackend        *backen
 
 
 
-static const gchar * const common_paper_sizes[] = {
+static const gchar const common_paper_sizes[][16] = {
   "na_letter",
   "na_legal",
   "iso_a4",
@@ -180,95 +181,78 @@ custom_paper_get_filename (void)
   return filename;
 }
 
-static gboolean
-scan_double (gchar   **text, 
-	     gdouble  *value, 
-	     gboolean  last)
-{
-  gchar *p, *e;
-
-  p = *text;
-  
-  *value = g_ascii_strtod (p, &e);
-  if (p == e)
-    return FALSE;
-
-  p = e;
-  if (!last)
-    {
-      while (g_ascii_isspace (*p))
-	p++;
-      if (*p++ != ',')
-	return FALSE;
-    }
-  *text = p;
-  return TRUE;
-}
-
 static void
 load_custom_papers (GtkListStore *store)
 {
+  GKeyFile *keyfile;
   gchar *filename;
-  gchar *contents;
-  
+  gchar **groups;
+  gsize n_groups, i;
+  gboolean load_ok;
+
   filename = custom_paper_get_filename ();
 
-  if (g_file_get_contents (filename, &contents, NULL, NULL))
+  keyfile = g_key_file_new ();
+  load_ok = g_key_file_load_from_file (keyfile, filename, 0, NULL);
+  g_free (filename);
+  if (!load_ok)
     {
-      gchar **lines = g_strsplit (contents, "\n", -1);
+      g_key_file_free (keyfile);
+      return;
+    }
+
+  groups = g_key_file_get_groups (keyfile, &n_groups);
+  for (i = 0; i < n_groups; ++i)
+    {
+      GError *error = NULL;
       gdouble w, h, top, bottom, left, right;
       GtkPaperSize *paper_size;
       GtkPageSetup *page_setup;
-      gchar *name, *p;
+      gchar *name;
       GtkTreeIter iter;
-      gint i;
 
-      for (i = 0; lines[i]; i++)
-	{
-	  name = lines[i];
-	  p = strchr (lines[i], ':');
-	  if (p == NULL)
-	    continue;
-	  *p++ = 0;
+      name = g_key_file_get_value (keyfile, groups[i], "Name", NULL);
+      if (!name)
+        continue;
 
-	  while (g_ascii_isspace (*p))
-	    p++;
-	  
-	  if (!scan_double (&p, &w, FALSE))
-	    continue;
-	  if (!scan_double (&p, &h, FALSE))
-	    continue;
-	  if (!scan_double (&p, &top, FALSE))
-	    continue;
-	  if (!scan_double (&p, &bottom, FALSE))
-	    continue;
-	  if (!scan_double (&p, &left, FALSE))
-	    continue;
-	  if (!scan_double (&p, &right, TRUE))
-	    continue;
+#define GET_DOUBLE(kf, name, v) \
+      v = g_key_file_get_double (kf, groups[i], name, &error); \
+      if (error != NULL) \
+        {\
+          g_error_free (error);\
+          continue;\
+        }
 
-	  page_setup = gtk_page_setup_new ();
-	  paper_size = gtk_paper_size_new_custom (name, name, w, h, GTK_UNIT_MM);
-	  gtk_page_setup_set_paper_size (page_setup, paper_size);
-	  gtk_paper_size_free (paper_size);
+      GET_DOUBLE (keyfile, "Width", w);
+      GET_DOUBLE (keyfile, "Height", h);
+      GET_DOUBLE (keyfile, "MarginTop", top);
+      GET_DOUBLE (keyfile, "MarginBottom", bottom);
+      GET_DOUBLE (keyfile, "MarginLeft", left);
+      GET_DOUBLE (keyfile, "MarginRight", right);
 
-	  gtk_page_setup_set_top_margin (page_setup, top, GTK_UNIT_MM);
-	  gtk_page_setup_set_bottom_margin (page_setup, bottom, GTK_UNIT_MM);
-	  gtk_page_setup_set_left_margin (page_setup, left, GTK_UNIT_MM);
-	  gtk_page_setup_set_right_margin (page_setup, right, GTK_UNIT_MM);
+#undef GET_DOUBLE
 
-	  gtk_list_store_append (store, &iter);
-	  gtk_list_store_set (store, &iter,
-			      0, page_setup, 
-			      -1);
+      page_setup = gtk_page_setup_new ();
+      paper_size = gtk_paper_size_new_custom (name, name, w, h, GTK_UNIT_MM);
+      gtk_page_setup_set_paper_size (page_setup, paper_size);
+      gtk_paper_size_free (paper_size);
 
-	  g_object_unref (page_setup);
-	}
-      
-      g_free (contents);
-      g_strfreev (lines);
+      gtk_page_setup_set_top_margin (page_setup, top, GTK_UNIT_MM);
+      gtk_page_setup_set_bottom_margin (page_setup, bottom, GTK_UNIT_MM);
+      gtk_page_setup_set_left_margin (page_setup, left, GTK_UNIT_MM);
+      gtk_page_setup_set_right_margin (page_setup, right, GTK_UNIT_MM);
+
+      gtk_list_store_append (store, &iter);
+      gtk_list_store_set (store, &iter,
+			  0, page_setup, 
+			  -1);
+
+      g_object_unref (page_setup);
+      g_free (name);
     }
-  g_free (filename);
+ 
+  g_strfreev (groups);
+  g_key_file_free (keyfile);
 }
 
 static void
@@ -276,10 +260,12 @@ save_custom_papers (GtkListStore *store)
 {
   GtkTreeModel *model = GTK_TREE_MODEL (store);
   GtkTreeIter iter;
-  GString *string;
-  gchar *filename;
+  GKeyFile *keyfile;
+  gchar *filename, *data;
+  gsize len;
+  gint i = 0;
 
-  string = g_string_new ("");
+  keyfile = g_key_file_new ();
   
   if (gtk_tree_model_get_iter_first (model, &iter))
     {
@@ -287,47 +273,37 @@ save_custom_papers (GtkListStore *store)
 	{
 	  GtkPaperSize *paper_size;
 	  GtkPageSetup *page_setup;
-	  gchar buffer[G_ASCII_DTOSTR_BUF_SIZE];
-	  
+	  gchar group[32];
+
+	  g_snprintf (group, sizeof (group), "Paper%u", i);
+
 	  gtk_tree_model_get (model, &iter, 0, &page_setup, -1);
 
 	  paper_size = gtk_page_setup_get_paper_size (page_setup);
-	  g_string_append_printf (string, "%s: ", gtk_paper_size_get_name (paper_size));
-	  
-	  g_ascii_dtostr (buffer, G_ASCII_DTOSTR_BUF_SIZE,
-			  gtk_page_setup_get_paper_width (page_setup, GTK_UNIT_MM));
-	  g_string_append (string, buffer);
-	  g_string_append (string, ", ");
-	  g_ascii_dtostr (buffer, G_ASCII_DTOSTR_BUF_SIZE,
-			  gtk_page_setup_get_paper_height (page_setup, GTK_UNIT_MM));
-	  g_string_append (string, buffer);
-	  g_string_append (string, ", ");
-	  g_ascii_dtostr (buffer, G_ASCII_DTOSTR_BUF_SIZE,
-			  gtk_page_setup_get_top_margin (page_setup, GTK_UNIT_MM));
-	  g_string_append (string, buffer);
-	  g_string_append (string, ", ");
-	  g_ascii_dtostr (buffer, G_ASCII_DTOSTR_BUF_SIZE, 
-			  gtk_page_setup_get_bottom_margin (page_setup, GTK_UNIT_MM));
-	  g_string_append (string, buffer);
-	  g_string_append (string, ", ");
-	  g_ascii_dtostr (buffer, G_ASCII_DTOSTR_BUF_SIZE,
-			  gtk_page_setup_get_left_margin (page_setup, GTK_UNIT_MM));
-	  g_string_append (string, buffer);
-	  g_string_append (string, ", ");
-	  g_ascii_dtostr (buffer, G_ASCII_DTOSTR_BUF_SIZE,
-			  gtk_page_setup_get_right_margin (page_setup, GTK_UNIT_MM));
-	  g_string_append (string, buffer);
+	  g_key_file_set_string (keyfile, group, "Name", gtk_paper_size_get_name (paper_size));
 
-	  g_string_append (string, "\n");
+	  g_key_file_set_double (keyfile, group, "Width",
+				 gtk_page_setup_get_paper_width (page_setup, GTK_UNIT_MM));
+	  g_key_file_set_double (keyfile, group, "Height",
+				 gtk_page_setup_get_paper_height (page_setup, GTK_UNIT_MM));
+	  g_key_file_set_double (keyfile, group, "MarginTop",
+				 gtk_page_setup_get_top_margin (page_setup, GTK_UNIT_MM));
+	  g_key_file_set_double (keyfile, group, "MarginBottom",
+				 gtk_page_setup_get_bottom_margin (page_setup, GTK_UNIT_MM));
+	  g_key_file_set_double (keyfile, group, "MarginLeft",
+				 gtk_page_setup_get_left_margin (page_setup, GTK_UNIT_MM));
+	  g_key_file_set_double (keyfile, group, "MarginRight",
+				 gtk_page_setup_get_right_margin (page_setup, GTK_UNIT_MM));
 	  
+	  ++i;
 	} while (gtk_tree_model_iter_next (model, &iter));
     }
 
   filename = custom_paper_get_filename ();
-  g_file_set_contents (filename, string->str, -1, NULL);
+  data = g_key_file_to_data (keyfile, &len, NULL);
+  g_file_set_contents (filename, data, len, NULL);
+  g_free (data);
   g_free (filename);
-  
-  g_string_free (string, TRUE);
 }
 
 static void
@@ -410,6 +386,12 @@ gtk_page_setup_unix_dialog_finalize (GObject *object)
       priv->page_setup_list = NULL;
     }
 
+  if (priv->custom_paper_list)
+    {
+      g_object_unref (priv->custom_paper_list);
+      priv->custom_paper_list = NULL;
+    }
+
   if (priv->print_settings)
     {
       g_object_unref (priv->print_settings);
@@ -514,6 +496,7 @@ printer_status_cb (GtkPrintBackend        *backend,
   gtk_list_store_set (priv->printer_list, iter,
                       PRINTER_LIST_COL_NAME, str,
                       -1);
+  g_free (str);
 }
 
 static void
@@ -1120,7 +1103,7 @@ populate_dialog (GtkPageSetupUnixDialog *ps_dialog)
 
   radio_button = create_radio_button (gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_button)),
 				      GTK_STOCK_ORIENTATION_REVERSE_PORTRAIT);
-  priv->reverse_landscape_radio = radio_button;
+  priv->reverse_portrait_radio = radio_button;
   gtk_table_attach (GTK_TABLE (table), radio_button,
 		    2, 3, 3, 4,
 		    GTK_EXPAND|GTK_FILL, 0, 0, 0);
@@ -1193,7 +1176,7 @@ get_orientation (GtkPageSetupUnixDialog *dialog)
     return GTK_PAGE_ORIENTATION_LANDSCAPE;
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->reverse_landscape_radio)))
     return GTK_PAGE_ORIENTATION_REVERSE_LANDSCAPE;
-  return GTK_PAGE_ORIENTATION_PORTRAIT;
+  return GTK_PAGE_ORIENTATION_REVERSE_PORTRAIT;
 }
 
 static void
@@ -1205,6 +1188,8 @@ set_orientation (GtkPageSetupUnixDialog *dialog,
   switch (orientation)
     {
     case GTK_PAGE_ORIENTATION_REVERSE_PORTRAIT:
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->reverse_portrait_radio), TRUE);
+      break;
     case GTK_PAGE_ORIENTATION_PORTRAIT:
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->portrait_radio), TRUE);
       break;
@@ -1669,6 +1654,7 @@ add_custom_paper (CustomPaperDialog *data)
   gtk_tree_view_set_cursor (GTK_TREE_VIEW (data->treeview), path,
 			    data->text_column, TRUE);
   gtk_tree_path_free (path);
+  g_free (name);
 }
 
 static void
