@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "gnu"; tab-width: 8 -*- */
 /* gtkpathbar.c
  * Copyright (C) 2004  Red Hat, Inc.,  Jonathan Blandford <jrb@gnome.org>
  *
@@ -116,6 +117,16 @@ static void gtk_path_bar_update_button_appearance (GtkPathBar       *path_bar,
 						   ButtonData       *button_data,
 						   gboolean          current_dir);
 
+static void
+on_slider_unmap (GtkWidget  *widget,
+		 GtkPathBar *path_bar)
+{
+  if (path_bar->timer &&
+      ((widget == path_bar->up_slider_button && path_bar->scrolling_up) ||
+       (widget == path_bar->down_slider_button && path_bar->scrolling_down)))
+    gtk_path_bar_stop_scrolling (path_bar);
+}
+
 static GtkWidget *
 get_slider_button (GtkPathBar  *path_bar,
 		   GtkArrowType arrow_type)
@@ -130,6 +141,9 @@ get_slider_button (GtkPathBar  *path_bar,
   gtk_container_add (GTK_CONTAINER (path_bar), button);
   gtk_widget_show_all (button);
 
+  g_signal_connect (G_OBJECT (button), "unmap",
+		    G_CALLBACK (on_slider_unmap), path_bar);
+
   gtk_widget_pop_composite_child ();
 
   return button;
@@ -693,7 +707,10 @@ gtk_path_bar_scroll_down (GtkWidget *button, GtkPathBar *path_bar)
 	}
     }
 
-  space_needed = BUTTON_DATA (down_button->data)->button->allocation.width + path_bar->spacing;
+  /* We check if down_button might be NULL in cases where the pathbar's horizontal size is smaller
+   * than the button and it doesn't get displayed. e.g., on Windows it might be "My Documents and Settings"
+   */
+  space_needed = down_button ? BUTTON_DATA (down_button->data)->button->allocation.width : 0 + path_bar->spacing;
   if (direction == GTK_TEXT_DIR_RTL)
     space_available = path_bar->down_slider_button->allocation.x - GTK_WIDGET (path_bar)->allocation.x;
   else
@@ -1658,3 +1675,6 @@ _gtk_path_bar_down (GtkPathBar *path_bar)
 	}
     }
 }
+
+#define __GTK_PATH_BAR_C__
+#include "gtkaliasdef.c"
