@@ -43,11 +43,13 @@ struct _GtkPrinterPrivate
   gchar *description;
   gchar *icon_name;
 
-  guint is_active: 1;
-  guint is_new: 1;
-  guint is_virtual : 1;
-  guint is_default : 1;
-  guint has_details: 1;
+  guint is_active   : 1;
+  guint is_new      : 1;
+  guint is_virtual  : 1;
+  guint is_default  : 1;
+  guint has_details : 1;
+  guint accepts_pdf : 1;
+  guint accepts_ps  : 1;
 
   gchar *state_message;  
   gint job_count;
@@ -68,7 +70,9 @@ enum {
   PROP_STATE_MESSAGE,
   PROP_LOCATION,
   PROP_ICON_NAME,
-  PROP_JOB_COUNT
+  PROP_JOB_COUNT,
+  PROP_ACCEPTS_PDF,
+  PROP_ACCEPTS_PS
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -130,6 +134,20 @@ gtk_printer_class_init (GtkPrinterClass *class)
 							 P_("FALSE if this represents a real hardware printer"),
 							 FALSE,
 							 GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property (G_OBJECT_CLASS (class),
+                                   PROP_ACCEPTS_PDF,
+                                   g_param_spec_boolean ("accepts-pdf",
+							 P_("Accepts PDF"),
+							 P_("TRUE if this printer can accept PDF"),
+							 TRUE,
+							 GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property (G_OBJECT_CLASS (class),
+                                   PROP_ACCEPTS_PS,
+                                   g_param_spec_boolean ("accepts-ps",
+							 P_("Accepts PostScript"),
+							 P_("TRUE if this printer can accept PostScript"),
+							 TRUE,
+							 GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (G_OBJECT_CLASS (class),
                                    PROP_STATE_MESSAGE,
                                    g_param_spec_string ("state-message",
@@ -197,6 +215,8 @@ gtk_printer_init (GtkPrinter *printer)
   priv->is_active = TRUE;
   priv->is_new = TRUE;
   priv->has_details = FALSE;
+  priv->accepts_pdf = TRUE;
+  priv->accepts_ps = TRUE;
 
   priv->state_message = NULL;  
   priv->job_count = 0;
@@ -243,6 +263,14 @@ gtk_printer_set_property (GObject         *object,
       priv->is_virtual = g_value_get_boolean (value);
       break;
 
+    case PROP_ACCEPTS_PDF:
+      priv->accepts_pdf = g_value_get_boolean (value);
+      break;
+
+    case PROP_ACCEPTS_PS:
+      priv->accepts_ps = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -290,6 +318,15 @@ gtk_printer_get_property (GObject    *object,
     case PROP_JOB_COUNT:
       g_value_set_int (value, priv->job_count);
       break;
+    case PROP_IS_VIRTUAL:
+      g_value_set_boolean (value, priv->is_virtual);
+      break;
+    case PROP_ACCEPTS_PDF:
+      g_value_set_boolean (value, priv->accepts_pdf);
+      break;
+    case PROP_ACCEPTS_PS:
+      g_value_set_boolean (value, priv->accepts_ps);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -607,6 +644,44 @@ gtk_printer_is_virtual (GtkPrinter *printer)
   return printer->priv->is_virtual;
 }
 
+/**
+ * gtk_printer_accepts_pdf:
+ * @printer: a #GtkPrinter
+ *
+ * Returns whether the printer accepts input in
+ * PDF format.  
+ *
+ * Return value: %TRUE if @printer accepts PDF
+ *
+ * Since: 2.10
+ */
+gboolean 
+gtk_printer_accepts_pdf (GtkPrinter *printer)
+{ 
+  g_return_val_if_fail (GTK_IS_PRINTER (printer), TRUE);
+  
+  return printer->priv->accepts_pdf;
+}
+
+/**
+ * gtk_printer_accepts_ps:
+ * @printer: a #GtkPrinter
+ *
+ * Returns whether the printer accepts input in
+ * PostScript format.  
+ *
+ * Return value: %TRUE if @printer accepts PostScript
+ *
+ * Since: 2.10
+ */
+gboolean 
+gtk_printer_accepts_ps (GtkPrinter *printer)
+{ 
+  g_return_val_if_fail (GTK_IS_PRINTER (printer), TRUE);
+  
+  return printer->priv->accepts_ps;
+}
+
 gboolean
 gtk_printer_is_new (GtkPrinter *printer)
 {
@@ -660,12 +735,13 @@ _gtk_printer_request_details (GtkPrinter *printer)
 }
 
 GtkPrinterOptionSet *
-_gtk_printer_get_options (GtkPrinter       *printer,
-			  GtkPrintSettings *settings,
-			  GtkPageSetup     *page_setup)
+_gtk_printer_get_options (GtkPrinter           *printer,
+			  GtkPrintSettings     *settings,
+			  GtkPageSetup         *page_setup,
+			  GtkPrintCapabilities  capabilities)
 {
   GtkPrintBackendClass *backend_class = GTK_PRINT_BACKEND_GET_CLASS (printer->priv->backend);
-  return backend_class->printer_get_options (printer, settings, page_setup);
+  return backend_class->printer_get_options (printer, settings, page_setup, capabilities);
 }
 
 gboolean
@@ -696,14 +772,16 @@ _gtk_printer_prepare_for_print (GtkPrinter       *printer,
 }
 
 cairo_surface_t *
-_gtk_printer_create_cairo_surface (GtkPrinter *printer,
-				   gdouble     width, 
-				   gdouble     height,
-				   gint        cache_fd)
+_gtk_printer_create_cairo_surface (GtkPrinter       *printer,
+				   GtkPrintSettings *settings,
+				   gdouble           width, 
+				   gdouble           height,
+				   GIOChannel       *cache_io)
 {
   GtkPrintBackendClass *backend_class = GTK_PRINT_BACKEND_GET_CLASS (printer->priv->backend);
 
-  return backend_class->printer_create_cairo_surface (printer, width, height, cache_fd);
+  return backend_class->printer_create_cairo_surface (printer, settings,
+						      width, height, cache_io);
 }
 
 GList  *
@@ -734,8 +812,21 @@ _gtk_printer_get_capabilities (GtkPrinter *printer)
   return backend_class->printer_get_capabilities (printer);
 }
 
+/**
+ * gtk_printer_compare:
+ * @a: a #GtkPrinter
+ * @b: another #GtkPrinter
+ *
+ * Compares two printers.
+ * 
+ * Return value: 0 if the printer match, a negative value if @a &lt; @b, 
+ *   or a positive value if @a &gt; @b
+ *
+ * Since: 2.10
+ */
 gint
-gtk_printer_compare (GtkPrinter *a, GtkPrinter *b)
+gtk_printer_compare (GtkPrinter *a, 
+                     GtkPrinter *b)
 {
   const char *name_a, *name_b;
   
@@ -753,5 +844,172 @@ gtk_printer_compare (GtkPrinter *a, GtkPrinter *b)
     return g_ascii_strcasecmp (name_a, name_b);
 }
 
+
+typedef struct 
+{
+  GList *backends;
+  GtkPrinterFunc func;
+  gpointer data;
+  GDestroyNotify destroy;
+  GMainLoop *loop;
+} PrinterList;
+
+static void list_done_cb (GtkPrintBackend *backend, 
+			  PrinterList     *printer_list);
+
+static void
+stop_enumeration (PrinterList *printer_list)
+{
+  GList *list, *next;
+  GtkPrintBackend *backend;
+
+  for (list = printer_list->backends; list; list = next)
+    {
+      next = list->next;
+      backend = GTK_PRINT_BACKEND (list->data);
+      list_done_cb (backend, printer_list);
+    }
+}
+
+static void 
+free_printer_list (PrinterList *printer_list)
+{
+  if (printer_list->destroy)
+    printer_list->destroy (printer_list->data);
+
+  if (printer_list->loop)
+    {    
+      g_main_loop_quit (printer_list->loop);
+      g_main_loop_unref (printer_list->loop);
+    }
+
+  g_free (printer_list);
+}
+
+static gboolean
+list_added_cb (GtkPrintBackend *backend, 
+	       GtkPrinter      *printer, 
+	       PrinterList     *printer_list)
+{
+  if (printer_list->func (printer, printer_list->data))
+    {
+      stop_enumeration (printer_list);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+list_done_cb (GtkPrintBackend *backend, 
+	      PrinterList     *printer_list)
+{
+  printer_list->backends = g_list_remove (printer_list->backends, backend);
+  
+  g_signal_handlers_disconnect_by_func (backend, list_added_cb, printer_list);
+  g_signal_handlers_disconnect_by_func (backend, list_done_cb, printer_list);
+  
+  gtk_print_backend_destroy (backend);
+  g_object_unref (backend);
+
+  if (printer_list->backends == NULL)
+    free_printer_list (printer_list);
+}
+
+static gboolean
+list_printers_init (PrinterList     *printer_list,
+		    GtkPrintBackend *backend)
+{
+  GList *list, *node;
+
+  list = gtk_print_backend_get_printer_list (backend);
+
+  for (node = list; node != NULL; node = node->next)
+    {
+      if (list_added_cb (backend, node->data, printer_list))
+        {
+          g_list_free (list);
+          return TRUE;
+        }
+    }
+
+  g_list_free (list);
+
+  if (gtk_print_backend_printer_list_is_done (backend))
+    {
+      printer_list->backends = g_list_remove (printer_list->backends, backend);
+      gtk_print_backend_destroy (backend);
+      g_object_unref (backend);
+    }
+  else
+    {
+      g_signal_connect (backend, "printer-added", 
+			(GCallback) list_added_cb, 
+			printer_list);
+      g_signal_connect (backend, "printer-list-done", 
+			(GCallback) list_done_cb, 
+			printer_list);
+    }
+
+  return FALSE;
+}
+
+/**
+ * gtk_enumerate_printers:
+ * @func: a function to call for each printer
+ * @data: user data to pass to @func
+ * @destroy: function to call if @data is no longer needed
+ * @wait: if %TRUE, wait in a recursive mainloop until
+ *    all printers are enumerated; otherwise return early
+ *
+ * Calls a function for all #GtkPrinter<!-- -->s. 
+ * If @func returns %TRUE, the enumeration is stopped.
+ *
+ * Since: 2.10
+ */
+void
+gtk_enumerate_printers (GtkPrinterFunc func,
+			gpointer       data,
+			GDestroyNotify destroy,
+			gboolean       wait)
+{
+  PrinterList *printer_list;
+  GList *node, *next;
+  GtkPrintBackend *backend;
+
+  printer_list = g_new0 (PrinterList, 1);
+
+  printer_list->func = func;
+  printer_list->data = data;
+  printer_list->destroy = destroy;
+
+  if (g_module_supported ())
+    printer_list->backends = gtk_print_backend_load_modules ();
+  
+  if (printer_list->backends == NULL)
+    {
+      free_printer_list (printer_list);
+      return;
+    }
+
+  for (node = printer_list->backends; node != NULL; node = next)
+    {
+      next = node->next;
+      backend = GTK_PRINT_BACKEND (node->data);
+      if (list_printers_init (printer_list, backend))
+        return;
+    }
+
+  if (wait && printer_list->backends)
+    {
+      printer_list->loop = g_main_loop_new (NULL, FALSE);
+
+      GDK_THREADS_LEAVE ();  
+      g_main_loop_run (printer_list->loop);
+      GDK_THREADS_ENTER ();  
+    }
+}
+
+
 #define __GTK_PRINTER_C__
 #include "gtkaliasdef.c"
