@@ -475,6 +475,7 @@ gtk_print_job_get_surface (GtkPrintJob  *job,
 			 &tmp_error);
   if (fd == -1)
     {
+      g_free (filename);
       g_propagate_error (error, tmp_error);
       return NULL;
     }
@@ -485,7 +486,8 @@ gtk_print_job_get_surface (GtkPrintJob  *job,
   /* If we are debugging printing don't delete the tmp files */
   if (!(gtk_debug_flags & GTK_DEBUG_PRINTING))
 #endif /* G_ENABLE_DEBUG */
-  unlink (filename);
+  g_unlink (filename);
+  g_free (filename);
 
   paper_size = gtk_page_setup_get_paper_size (priv->page_setup);
   width = gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS);
@@ -650,7 +652,7 @@ gtk_print_job_get_property (GObject    *object,
 /**
  * gtk_print_job_send:
  * @job: a GtkPrintJob
- * @callback: function to call when the job completes or an error occures
+ * @callback: function to call when the job completes or an error occurs
  * @user_data: user data that gets passed to @callback
  * @dnotify: destroy notify for @user_data
  * 
@@ -693,6 +695,9 @@ gtk_print_capabilities_get_type (void)
         { GTK_PRINT_CAPABILITY_COLLATE, "GTK_PRINT_CAPABILITY_COLLATE", "collate" },
         { GTK_PRINT_CAPABILITY_REVERSE, "GTK_PRINT_CAPABILITY_REVERSE", "reverse" },
         { GTK_PRINT_CAPABILITY_SCALE, "GTK_PRINT_CAPABILITY_SCALE", "scale" },
+        { GTK_PRINT_CAPABILITY_GENERATE_PDF, "GTK_PRINT_CAPABILITY_GENERATE_PDF", "generate-pdf" },
+        { GTK_PRINT_CAPABILITY_GENERATE_PS, "GTK_PRINT_CAPABILITY_GENERATE_PS", "generate-ps" },
+        { GTK_PRINT_CAPABILITY_PREVIEW, "GTK_PRINT_CAPABILITY_PREVIEW", "preview" },
         { 0, NULL, NULL }
       };
 
