@@ -57,12 +57,12 @@ static void gtk_ruler_get_property  (GObject        *object,
 
 static const GtkRulerMetric ruler_metrics[] =
 {
-  { NULL, NULL, 1.0, { 1, 2, 5, 10, 25, 50, 100, 250, 500, 1000 }, { 1, 5, 10, 50, 100 }},
-  { NULL, NULL, 72.0, { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512 }, { 1, 2, 4, 8, 16 }},
-  { NULL, NULL, 28.35, { 1, 2, 5, 10, 25, 50, 100, 250, 500, 1000 }, { 1, 5, 10, 50, 100 }},
+  { "Pixel", "Pi", 1.0, { 1, 2, 5, 10, 25, 50, 100, 250, 500, 1000 }, { 1, 5, 10, 50, 100 }},
+  { "Inches", "In", 72.0, { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512 }, { 1, 2, 4, 8, 16 }},
+  { "Centimeters", "Cn", 28.35, { 1, 2, 5, 10, 25, 50, 100, 250, 500, 1000 }, { 1, 5, 10, 50, 100 }},
 };
 
-G_DEFINE_TYPE (GtkRuler, gtk_ruler, GTK_TYPE_WIDGET);
+G_DEFINE_TYPE (GtkRuler, gtk_ruler, GTK_TYPE_WIDGET)
 
 static void
 gtk_ruler_class_init (GtkRulerClass *class)
@@ -385,10 +385,16 @@ gtk_ruler_unrealize (GtkWidget *widget)
   GtkRuler *ruler = GTK_RULER (widget);
 
   if (ruler->backing_store)
-    g_object_unref (ruler->backing_store);
+    {
+      g_object_unref (ruler->backing_store);
+      ruler->backing_store = NULL;
+    }
 
-  ruler->backing_store = NULL;
-  ruler->non_gr_exp_gc = NULL;
+  if (ruler->non_gr_exp_gc)
+    {
+      g_object_unref (ruler->non_gr_exp_gc);
+      ruler->non_gr_exp_gc = NULL;
+    }
 
   if (GTK_WIDGET_CLASS (gtk_ruler_parent_class)->unrealize)
     (* GTK_WIDGET_CLASS (gtk_ruler_parent_class)->unrealize) (widget);
