@@ -228,7 +228,7 @@ gtk_tool_button_class_init (GtkToolButtonClass *klass)
 							GTK_PARAM_READWRITE));
 
   /**
-   * GtkButton::icon-spacing:
+   * GtkButton:icon-spacing:
    * 
    * Spacing in pixels between the icon and label.
    * 
@@ -405,7 +405,8 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 			    NULL);
 	    }
 	}
-      else if (button->priv->stock_id)
+      else if (button->priv->stock_id && 
+	       gtk_icon_factory_lookup_default (button->priv->stock_id))
 	{
 	  icon = gtk_image_new_from_stock (button->priv->stock_id, icon_size);
 	  gtk_widget_show (icon);
