@@ -25,40 +25,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gtkvscrollbar.h"
 #include "gdk/gdkkeysyms.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
-static void     gtk_vscrollbar_class_init       (GtkVScrollbarClass *klass);
-static void     gtk_vscrollbar_init             (GtkVScrollbar      *vscrollbar);
-
-GType
-gtk_vscrollbar_get_type (void)
-{
-  static GType vscrollbar_type = 0;
-  
-  if (!vscrollbar_type)
-    {
-      static const GTypeInfo vscrollbar_info =
-      {
-        sizeof (GtkVScrollbarClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) gtk_vscrollbar_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-        sizeof (GtkVScrollbar),
-	0,		/* n_preallocs */
-        (GInstanceInitFunc) gtk_vscrollbar_init,
-      };
-      
-      vscrollbar_type =
-	g_type_register_static (GTK_TYPE_SCROLLBAR, "GtkVScrollbar",
-				&vscrollbar_info, 0);
-    }
-  
-  return vscrollbar_type;
-}
+G_DEFINE_TYPE (GtkVScrollbar, gtk_vscrollbar, GTK_TYPE_SCROLLBAR)
 
 static void
 gtk_vscrollbar_class_init (GtkVScrollbarClass *class)
@@ -87,3 +60,6 @@ gtk_vscrollbar_new (GtkAdjustment *adjustment)
   
   return vscrollbar;
 }
+
+#define __GTK_VSCROLLBAR_C__
+#include "gtkaliasdef.c"
